﻿// # C3 Butterfly Execution Flow
// _Demonstrates a Butterfly Caller/Callee flow chart for program execution_.

// ## Example data sets for function-level control flow graph.

// Function records contain an `id`, their `name`, and the name of the `module` they belong to.
interface FunctionRecord {
    id: number;
    name: string;
    module: string; // Not used in this example
}
// Function call links records contain the function id for the `source` and `target` of the call.
// `source` is the **caller** and `target` is the **callee**.
// `value` is the total amount of time for the function call (and any nested calls).
interface CFGLinkRecord {
    source: number;
    target: number;
    value: number;
}
// Example data sets
var cfg_datasets = {
    libpthread: {
        functions: [{ "id": 255, "name": "start_thread", "module": "libpthread-2.17.so" }, { "id": 318, "name": "libpthread-2.17.so@56e0", "module": "libpthread-2.17.so" }, { "id": 321, "name": "libpthread-2.17.so@55c0", "module": "libpthread-2.17.so" }, { "id": 325, "name": "(__kmp_launch_worker, void)", "module": "libomp.so" }, { "id": 345, "name": "(__kmp_itt_thr_name_set_init_3_0, const char, int)", "module": "libomp.so" }, { "id": 346, "name": "(__kmp_itt_thread_set_name_init_3_0, const char)", "module": "libomp.so" }, { "id": 350, "name": "libomp.so@17270", "module": "libomp.so" }, { "id": 352, "name": "__GI___pthread_mutex_lock", "module": "libpthread-2.17.so" }, { "id": 354, "name": "__lll_lock_wait", "module": "libpthread-2.17.so" }, { "id": 366, "name": "libomp.so@16ee0", "module": "libomp.so" }, { "id": 477, "name": "libomp.so@16e90", "module": "libomp.so" }, { "id": 479, "name": "__pthread_mutex_unlock", "module": "libpthread-2.17.so" }, { "id": 481, "name": "__lll_unlock_wake", "module": "libpthread-2.17.so" }, { "id": 502, "name": "(__kmp_set_stack_info, int, kmp_info)", "module": "libomp.so" }, { "id": 503, "name": "libomp.so@16c90", "module": "libomp.so" }, { "id": 506, "name": "libpthread-2.17.so@5690", "module": "libpthread-2.17.so" }, { "id": 508, "name": "libomp.so@168a0", "module": "libomp.so" }, { "id": 510, "name": "libpthread-2.17.so@5700", "module": "libpthread-2.17.so" }, { "id": 512, "name": "pthread_getaffinity_np@@GLIBC_2.3.4", "module": "libpthread-2.17.so" }, { "id": 521, "name": "libpthread-2.17.so@5540", "module": "libpthread-2.17.so" }, { "id": 524, "name": "libomp.so@17110", "module": "libomp.so" }, { "id": 527, "name": "libpthread-2.17.so@5430", "module": "libpthread-2.17.so" }, { "id": 530, "name": "__kmp_launch_thread", "module": "libomp.so" }, { "id": 541, "name": "__kmp_fork_barrier(int, int)", "module": "libomp.so" }, { "id": 547, "name": "__kmp_suspend_64", "module": "libomp.so" }, { "id": 555, "name": "libomp.so@16a30", "module": "libomp.so" }, { "id": 567, "name": "__pthread_mutex_unlock_usercnt", "module": "libpthread-2.17.so" }, { "id": 568, "name": "__pthread_enable_asynccancel", "module": "libpthread-2.17.so" }, { "id": 569, "name": "__pthread_disable_asynccancel", "module": "libpthread-2.17.so" }, { "id": 570, "name": "__pthread_mutex_cond_lock", "module": "libpthread-2.17.so" }, { "id": 583, "name": "__kmp_get_global_thread_id_reg", "module": "libomp.so" }, { "id": 603, "name": "__nptl_deallocate_tsd", "module": "libpthread-2.17.so" }, { "id": 606, "name": "libpthread-2.17.so@5620", "module": "libpthread-2.17.so" }, { "id": 615, "name": "libpthread-2.17.so@5770", "module": "libpthread-2.17.so" }, { "id": 782, "name": "libomp.so@16940", "module": "libomp.so" }, { "id": 805, "name": "__kmp_allocate_team", "module": "libomp.so" }, { "id": 810, "name": "__kmp_allocate_thread", "module": "libomp.so" }, { "id": 811, "name": "__kmp_create_monitor", "module": "libomp.so" }, { "id": 824, "name": "libomp.so@16860", "module": "libomp.so" }, { "id": 827, "name": "libpthread-2.17.so@5520", "module": "libpthread-2.17.so" }, { "id": 836, "name": "libpthread-2.17.so@57c0", "module": "libpthread-2.17.so" }, { "id": 844, "name": "libpthread-2.17.so@5790", "module": "libpthread-2.17.so" }, { "id": 846, "name": "do_clone.constprop.4", "module": "libpthread-2.17.so" }, { "id": 847, "name": "libpthread-2.17.so@5420", "module": "libpthread-2.17.so" }, { "id": 910, "name": "__kmp_create_worker", "module": "libomp.so" }, { "id": 927, "name": "main", "module": "a.out" }, { "id": 945, "name": "a.out@400600", "module": "a.out" }, { "id": 950, "name": "__GI_exit", "module": "libc-2.17.so" }, { "id": 951, "name": "__run_exit_handlers", "module": "libc-2.17.so" }, { "id": 953, "name": "_dl_fini", "module": "ld-2.17.so" }, { "id": 960, "name": "__kmp_internal_end_dtor", "module": "libomp.so" }, { "id": 970, "name": "(__kmp_reap_thread, kmp_info, int)", "module": "libomp.so" }, { "id": 977, "name": "__kmp_str_format", "module": "libomp.so" }, { "id": 997, "name": "libomp.so@16b90", "module": "libomp.so" }, { "id": 1012, "name": "libomp.so@170b0", "module": "libomp.so" }, { "id": 1015, "name": "_pthread_cleanup_push", "module": "libpthread-2.17.so" }, { "id": 1016, "name": "__pthread_cleanup_pop", "module": "libpthread-2.17.so" }, { "id": 1017, "name": "__free_tcb", "module": "libpthread-2.17.so" }, { "id": 1028, "name": "libomp.so@171b0", "module": "libomp.so" }, { "id": 1034, "name": "__kmp_cleanup", "module": "libomp.so" }, { "id": 1041, "name": "__kmp_runtime_destroy", "module": "libomp.so" }, { "id": 1042, "name": "__kmp_itt_destroy()", "module": "libomp.so" }, { "id": 1046, "name": "libomp.so@171f0", "module": "libomp.so" }, { "id": 1092, "name": "__do_global_dtors_aux", "module": "libpthread-2.17.so" }, { "id": 1093, "name": "libpthread-2.17.so@5880", "module": "libpthread-2.17.so" }, { "id": 1095, "name": "deregister_tm_clones", "module": "libpthread-2.17.so" }, { "id": 1096, "name": "_fini", "module": "libpthread-2.17.so" }, { "id": 1401, "name": "_dl_init", "module": "ld-2.17.so" }, { "id": 1402, "name": "_init", "module": "libpthread-2.17.so" }, { "id": 1403, "name": "__pthread_initialize_minimal", "module": "libpthread-2.17.so" }, { "id": 1405, "name": "__libc_sigaction", "module": "libpthread-2.17.so" }, { "id": 1412, "name": "libpthread-2.17.so@57f0", "module": "libpthread-2.17.so" }, { "id": 1415, "name": "libpthread-2.17.so@54d0", "module": "libpthread-2.17.so" }, { "id": 1418, "name": "libpthread-2.17.so@5440", "module": "libpthread-2.17.so" }, { "id": 1424, "name": "libpthread-2.17.so@5780", "module": "libpthread-2.17.so" }, { "id": 1426, "name": "libpthread-2.17.so@56d0", "module": "libpthread-2.17.so" }, { "id": 1430, "name": "frame_dummy", "module": "libpthread-2.17.so" }, { "id": 1455, "name": "a.out@4005d0", "module": "a.out" }, { "id": 1482, "name": "(__kmp_do_serial_initialize, void)", "module": "libomp.so" }, { "id": 1484, "name": "__kmp_register_library_startup()", "module": "libomp.so" }, { "id": 1488, "name": "ptmalloc_init.part.8", "module": "libc-2.17.so" }, { "id": 1489, "name": "_dl_addr", "module": "libc-2.17.so" }, { "id": 1535, "name": "sysconf", "module": "libc-2.17.so" }, { "id": 1569, "name": "__kmp_env_initialize(const char)", "module": "libomp.so" }, { "id": 1570, "name": "(__kmp_stg_init, void)", "module": "libomp.so" }, { "id": 1572, "name": "get_phys_pages", "module": "libc-2.17.so" }, { "id": 1574, "name": "_IO_new_fopen", "module": "libc-2.17.so" }, { "id": 1576, "name": "_IO_file_init@@GLIBC_2.2.5", "module": "libc-2.17.so" }, { "id": 1577, "name": "__GI__IO_link_in", "module": "libc-2.17.so" }, { "id": 1578, "name": "_pthread_cleanup_push_defer", "module": "libpthread-2.17.so" }, { "id": 1579, "name": "_pthread_cleanup_pop_restore", "module": "libpthread-2.17.so" }, { "id": 1646, "name": "__GI_sscanf", "module": "libc-2.17.so" }, { "id": 1647, "name": "vsscanf", "module": "libc-2.17.so" }, { "id": 1649, "name": "_IO_vfscanf", "module": "libc-2.17.so" }, { "id": 1653, "name": "fclose@@GLIBC_2.2.5", "module": "libc-2.17.so" }, { "id": 1654, "name": "__GI__IO_un_link", "module": "libc-2.17.so" }, { "id": 1757, "name": "(__kmp_launch_monitor, void)", "module": "libomp.so" }, { "id": 1758, "name": "(__kmp_itt_thr_ignore_init_3_0, void)", "module": "libomp.so" }, { "id": -1, "name": "<Unknown>", "module": "<Unknown>" }],
        links: [{ "source": 345, "target": 346, "value": 170 }, { "source": 346, "target": 350, "value": 110 }, { "source": 1570, "target": 782, "value": 134 }, { "source": 1569, "target": 1570, "value": 134 }, { "source": 782, "target": 1535, "value": 134 }, { "source": 1572, "target": 1653, "value": 44 }, { "source": 1535, "target": 1572, "value": 134 }, { "source": 1654, "target": 1579, "value": 16 }, { "source": 1653, "target": 1654, "value": 44 }, { "source": 1654, "target": 1578, "value": 28 }, { "source": 1649, "target": 1579, "value": 16 }, { "source": 1647, "target": 1649, "value": 44 }, { "source": 1646, "target": 1647, "value": 44 }, { "source": 1572, "target": 1646, "value": 44 }, { "source": 1649, "target": 1578, "value": 28 }, { "source": 1572, "target": 1574, "value": 46 }, { "source": 1577, "target": 1579, "value": 16 }, { "source": 1576, "target": 1577, "value": 46 }, { "source": 1574, "target": 1576, "value": 46 }, { "source": 1577, "target": 1578, "value": 30 }, { "source": 977, "target": 366, "value": 100 }, { "source": 1482, "target": 1484, "value": 100 }, { "source": 583, "target": 1482, "value": 100 }, { "source": 945, "target": 583, "value": 100 }, { "source": 927, "target": 945, "value": 100 }, { "source": 1455, "target": 927, "value": 100 }, { "source": 1484, "target": 977, "value": 100 }, { "source": 366, "target": 1488, "value": 100 }, { "source": 1489, "target": 479, "value": 48 }, { "source": 1488, "target": 1489, "value": 100 }, { "source": 1489, "target": 352, "value": 52 }, { "source": 1401, "target": 1430, "value": 32 }, { "source": 1403, "target": 1426, "value": 838 }, { "source": 1402, "target": 1403, "value": 1944 }, { "source": 1401, "target": 1402, "value": 1952 }, { "source": 1403, "target": 1424, "value": 58 }, { "source": 1403, "target": 1418, "value": 62 }, { "source": 1403, "target": 1415, "value": 358 }, { "source": 1403, "target": 1412, "value": 62 }, { "source": 1403, "target": 1405, "value": 266 }, { "source": 325, "target": 345, "value": 170 }, { "source": 255, "target": 325, "value": 1418 }, { "source": 477, "target": 481, "value": 40 }, { "source": 255, "target": 1757, "value": 500 }, { "source": 524, "target": 527, "value": 802 }, { "source": 502, "target": 524, "value": 688 }, { "source": 1757, "target": 502, "value": 478 }, { "source": 512, "target": 521, "value": 186 }, { "source": 508, "target": 512, "value": 346 }, { "source": 502, "target": 508, "value": 892 }, { "source": 508, "target": 510, "value": 546 }, { "source": 503, "target": 506, "value": 114 }, { "source": 502, "target": 503, "value": 32 }, { "source": 1758, "target": 481, "value": 22 }, { "source": 1757, "target": 1758, "value": 22 }, { "source": 255, "target": 321, "value": 298 }, { "source": 255, "target": 318, "value": 186 }, { "source": 255, "target": 615, "value": 186 }, { "source": 255, "target": 506, "value": 24 }, { "source": 255, "target": 606, "value": 324 }, { "source": 255, "target": 603, "value": 4062 }, { "source": 325, "target": 530, "value": 114 }, { "source": 530, "target": 541, "value": 114 }, { "source": 547, "target": 477, "value": 20 }, { "source": 541, "target": 547, "value": 114 }, { "source": 555, "target": 570, "value": 38 }, { "source": 547, "target": 555, "value": 94 }, { "source": 555, "target": 569, "value": 6 }, { "source": 555, "target": 568, "value": 12 }, { "source": 555, "target": 567, "value": 38 }, { "source": 325, "target": 502, "value": 1134 }, { "source": 346, "target": 481, "value": 60 }, { "source": 350, "target": 354, "value": 110 }, { "source": 950, "target": 951, "value": 754 }, { "source": 951, "target": 953, "value": 754 }, { "source": 953, "target": 1096, "value": 6 }, { "source": 1092, "target": 1095, "value": 18 }, { "source": 953, "target": 1092, "value": 224 }, { "source": 1092, "target": 1093, "value": 182 }, { "source": 953, "target": 960, "value": 424 }, { "source": 960, "target": 1034, "value": 100 }, { "source": 1034, "target": 1041, "value": 100 }, { "source": 1046, "target": 479, "value": 48 }, { "source": 1042, "target": 1046, "value": 100 }, { "source": 1041, "target": 1042, "value": 100 }, { "source": 1046, "target": 352, "value": 52 }, { "source": 1012, "target": 1017, "value": 188 }, { "source": 1028, "target": 1012, "value": 162 }, { "source": 960, "target": 1028, "value": 162 }, { "source": 1012, "target": 1016, "value": 20 }, { "source": 1012, "target": 569, "value": 48 }, { "source": 1012, "target": 568, "value": 44 }, { "source": 1012, "target": 1015, "value": 24 }, { "source": 960, "target": 970, "value": 162 }, { "source": 997, "target": 1012, "value": 162 }, { "source": 970, "target": 997, "value": 162 }, { "source": 953, "target": 479, "value": 48 }, { "source": 953, "target": 352, "value": 52 }, { "source": 910, "target": 524, "value": 48 }, { "source": 810, "target": 910, "value": 1290 }, { "source": 805, "target": 810, "value": 2020 }, { "source": 846, "target": 847, "value": 202 }, { "source": 824, "target": 846, "value": 458 }, { "source": 910, "target": 824, "value": 1218 }, { "source": 824, "target": 844, "value": 98 }, { "source": 824, "target": 836, "value": 1130 }, { "source": 824, "target": 827, "value": 106 }, { "source": 824, "target": 506, "value": 32 }, { "source": 910, "target": 503, "value": 24 }, { "source": 811, "target": 524, "value": 66 }, { "source": 810, "target": 811, "value": 730 }, { "source": 811, "target": 824, "value": 606 }, { "source": 811, "target": 503, "value": 58 }],
    },
    schedule: {
        functions: [{ "id": 0, "name": "__schedule", "module": "<.text.kernel>" }, { "id": 1, "name": "__context_tracking_task_switch", "module": "<.text.kernel>" }, { "id": 2, "name": "__switch_to", "module": "<.text.kernel>" }, { "id": 3, "name": "native_load_sp0", "module": "<.text.kernel>" }, { "id": 4, "name": "native_load_tls", "module": "<.text.kernel>" }, { "id": 5, "name": "native_write_msr_safe", "module": "<.text.kernel>" }, { "id": 8, "name": "finish_task_switch", "module": "<.text.kernel>" }, { "id": 9, "name": "vtime_common_task_switch", "module": "<.text.kernel>" }, { "id": 10, "name": "vtime_account_idle", "module": "<.text.kernel>" }, { "id": 11, "name": "get_vtime_delta", "module": "<.text.kernel>" }, { "id": 12, "name": "local_clock", "module": "<.text.kernel>" }, { "id": 13, "name": "sched_clock_cpu", "module": "<.text.kernel>" }, { "id": 14, "name": "sched_clock", "module": "<.text.kernel>" }, { "id": 15, "name": "native_sched_clock", "module": "<.text.kernel>" }, { "id": 16, "name": "native_read_tsc", "module": "<.text.kernel>" }, { "id": 17, "name": "arch_vtime_task_switch", "module": "<.text.kernel>" }, { "id": 18, "name": "_raw_spin_lock", "module": "<.text.kernel>" }, { "id": 19, "name": "_raw_spin_unlock", "module": "<.text.kernel>" }, { "id": 29, "name": "__vtime_account_system", "module": "<.text.kernel>" }, { "id": 30, "name": "account_system_time", "module": "<.text.kernel>" }, { "id": 31, "name": "cpuacct_account_field", "module": "<.text.kernel>" }, { "id": 32, "name": "acct_account_cputime", "module": "<.text.kernel>" }, { "id": 33, "name": "__acct_update_integrals", "module": "<.text.kernel>" }, { "id": 34, "name": "jiffies_to_timeval", "module": "<.text.kernel>" }, { "id": 36, "name": "rcu_eqs_enter", "module": "<.text.kernel>" }, { "id": 37, "name": "rcu_eqs_enter_common.isra.47", "module": "<.text.kernel>" }, { "id": 44, "name": "start_thread", "module": "libpthread-2.19.so" }, { "id": 47, "name": "system_call", "module": "<.text.kernel>" }, { "id": 52, "name": "rcu_eqs_exit", "module": "<.text.kernel>" }, { "id": 53, "name": "rcu_eqs_exit_common.isra.48", "module": "<.text.kernel>" }, { "id": 105, "name": "down_read", "module": "<.text.kernel>" }, { "id": 114, "name": "_raw_spin_lock_irqsave", "module": "<.text.kernel>" }, { "id": 120, "name": "_raw_spin_unlock_irqrestore", "module": "<.text.kernel>" }, { "id": 142, "name": "source_load", "module": "<.text.kernel>" }, { "id": 143, "name": "target_load", "module": "<.text.kernel>" }, { "id": 145, "name": "idle_cpu", "module": "<.text.kernel>" }, { "id": 149, "name": "update_rq_clock.part.63", "module": "<.text.kernel>" }, { "id": 152, "name": "update_curr", "module": "<.text.kernel>" }, { "id": 153, "name": "update_cfs_rq_blocked_load", "module": "<.text.kernel>" }, { "id": 154, "name": "account_entity_enqueue", "module": "<.text.kernel>" }, { "id": 155, "name": "update_cfs_shares", "module": "<.text.kernel>" }, { "id": 158, "name": "rb_insert_color", "module": "<.text.kernel>" }, { "id": 159, "name": "hrtick_update", "module": "<.text.kernel>" }, { "id": 169, "name": "tracing_record_cmdline", "module": "<.text.kernel>" }, { "id": 170, "name": "tracing_is_on", "module": "<.text.kernel>" }, { "id": 171, "name": "ring_buffer_record_is_on", "module": "<.text.kernel>" }, { "id": 173, "name": "reschedule_interrupt", "module": "<.text.kernel>" }, { "id": 174, "name": "smp_reschedule_interrupt", "module": "<.text.kernel>" }, { "id": 175, "name": "native_apic_mem_write", "module": "<.text.kernel>" }, { "id": 176, "name": "scheduler_ipi", "module": "<.text.kernel>" }, { "id": 181, "name": "rcu_note_context_switch", "module": "<.text.kernel>" }, { "id": 182, "name": "_raw_spin_lock_irq", "module": "<.text.kernel>" }, { "id": 183, "name": "put_prev_task_fair", "module": "<.text.kernel>" }, { "id": 184, "name": "update_min_vruntime", "module": "<.text.kernel>" }, { "id": 185, "name": "cpuacct_charge", "module": "<.text.kernel>" }, { "id": 186, "name": "pick_next_task_fair", "module": "<.text.kernel>" }, { "id": 187, "name": "clear_buddies", "module": "<.text.kernel>" }, { "id": 188, "name": "set_next_entity", "module": "<.text.kernel>" }, { "id": 189, "name": "update_stats_wait_end", "module": "<.text.kernel>" }, { "id": 190, "name": "rb_next", "module": "<.text.kernel>" }, { "id": 191, "name": "rb_erase", "module": "<.text.kernel>" }, { "id": 192, "name": "probe_sched_switch", "module": ".text.simple_pt" }, { "id": 193, "name": "poor_mans_ptwrite_prev_pid", "module": ".text.simple_pt" }, { "id": 195, "name": "poor_mans_ptwrite_pid", "module": ".text.simple_pt" }, { "id": 197, "name": "ftrace_raw_event_switch_out", "module": ".text.simple_pt" }, { "id": 198, "name": "trace_event_buffer_lock_reserve", "module": "<.text.kernel>" }, { "id": 199, "name": "trace_buffer_lock_reserve", "module": "<.text.kernel>" }, { "id": 200, "name": "ring_buffer_lock_reserve", "module": "<.text.kernel>" }, { "id": 201, "name": "rb_reserve_next_event", "module": "<.text.kernel>" }, { "id": 202, "name": "trace_clock_local", "module": "<.text.kernel>" }, { "id": 203, "name": "ring_buffer_event_data", "module": "<.text.kernel>" }, { "id": 204, "name": "rb_event_data", "module": "<.text.kernel>" }, { "id": 205, "name": "tracing_generic_entry_update", "module": "<.text.kernel>" }, { "id": 206, "name": "filter_check_discard", "module": "<.text.kernel>" }, { "id": 207, "name": "trace_buffer_unlock_commit", "module": "<.text.kernel>" }, { "id": 208, "name": "ring_buffer_unlock_commit", "module": "<.text.kernel>" }, { "id": 209, "name": "rb_update_write_stamp", "module": "<.text.kernel>" }, { "id": 210, "name": "ftrace_trace_userstack", "module": "<.text.kernel>" }, { "id": 211, "name": "probe_sched_switch", "module": "<.text.kernel>" }, { "id": 212, "name": "perf_event_task_sched_out", "module": "<.text.kernel>" }, { "id": 213, "name": "vtime_account_system", "module": "<.text.kernel>" }, { "id": 214, "name": "schedule", "module": "<.text.kernel>" }, { "id": 219, "name": "deactivate_task", "module": "<.text.kernel>" }, { "id": 220, "name": "dequeue_task", "module": "<.text.kernel>" }, { "id": 221, "name": "dequeue_task_fair", "module": "<.text.kernel>" }, { "id": 222, "name": "dequeue_entity", "module": "<.text.kernel>" }, { "id": 223, "name": "account_entity_dequeue", "module": "<.text.kernel>" }, { "id": 224, "name": "ftrace_raw_event_switch_in", "module": ".text.simple_pt" }, { "id": 225, "name": "SyS_exit", "module": "<.text.kernel>" }, { "id": 226, "name": "do_exit", "module": "<.text.kernel>" }, { "id": 258, "name": "call_rwsem_down_read_failed", "module": "<.text.kernel>" }, { "id": 259, "name": "rwsem_down_read_failed", "module": "<.text.kernel>" }, { "id": 260, "name": "__calc_delta", "module": "<.text.kernel>" }, { "id": 261, "name": "idle_balance", "module": "<.text.kernel>" }, { "id": 262, "name": "update_blocked_averages", "module": "<.text.kernel>" }, { "id": 263, "name": "update_rq_clock", "module": "<.text.kernel>" }, { "id": 264, "name": "load_balance", "module": "<.text.kernel>" }, { "id": 265, "name": "find_busiest_group", "module": "<.text.kernel>" }, { "id": 266, "name": "update_group_power", "module": "<.text.kernel>" }, { "id": 267, "name": "msecs_to_jiffies", "module": "<.text.kernel>" }, { "id": 268, "name": "sched_cfs_period_timer", "module": "<.text.kernel>" }, { "id": 269, "name": "cpumask_next_and", "module": "<.text.kernel>" }, { "id": 270, "name": "find_next_bit", "module": "<.text.kernel>" }, { "id": 271, "name": "pick_next_task_stop", "module": "<.text.kernel>" }, { "id": 272, "name": "pick_next_task_rt", "module": "<.text.kernel>" }, { "id": 273, "name": "pick_next_task_idle", "module": "<.text.kernel>" }, { "id": 274, "name": "post_schedule_idle", "module": "<.text.kernel>" }, { "id": 275, "name": "idle_enter_fair", "module": "<.text.kernel>" }, { "id": 276, "name": "schedule_preempt_disabled", "module": "<.text.kernel>" }, { "id": 278, "name": "tick_nohz_idle_enter", "module": "<.text.kernel>" }, { "id": 279, "name": "set_cpu_sd_state_idle", "module": "<.text.kernel>" }, { "id": 280, "name": "__tick_nohz_idle_enter", "module": "<.text.kernel>" }, { "id": 281, "name": "ktime_get", "module": "<.text.kernel>" }, { "id": 282, "name": "read_tsc", "module": "<.text.kernel>" }, { "id": 283, "name": "sched_clock_idle_sleep_event", "module": "<.text.kernel>" }, { "id": 284, "name": "tick_nohz_stop_sched_tick", "module": "<.text.kernel>" }, { "id": 285, "name": "timekeeping_max_deferment", "module": "<.text.kernel>" }, { "id": 286, "name": "rcu_needs_cpu", "module": "<.text.kernel>" }, { "id": 287, "name": "rcu_cpu_has_callbacks", "module": "<.text.kernel>" }, { "id": 288, "name": "irq_work_needs_cpu", "module": "<.text.kernel>" }, { "id": 289, "name": "get_next_timer_interrupt", "module": "<.text.kernel>" }, { "id": 290, "name": "hrtimer_get_next_event", "module": "<.text.kernel>" }, { "id": 291, "name": "nohz_balance_enter_idle", "module": "<.text.kernel>" }, { "id": 292, "name": "calc_load_enter_idle", "module": "<.text.kernel>" }, { "id": 293, "name": "hrtimer_start", "module": "<.text.kernel>" }, { "id": 294, "name": "__hrtimer_start_range_ns", "module": "<.text.kernel>" }, { "id": 295, "name": "lock_hrtimer_base.isra.19", "module": "<.text.kernel>" }, { "id": 296, "name": "__remove_hrtimer", "module": "<.text.kernel>" }, { "id": 297, "name": "timerqueue_del", "module": "<.text.kernel>" }, { "id": 298, "name": "hrtimer_force_reprogram", "module": "<.text.kernel>" }, { "id": 299, "name": "tick_program_event", "module": "<.text.kernel>" }, { "id": 300, "name": "clockevents_program_event", "module": "<.text.kernel>" }, { "id": 301, "name": "lapic_next_deadline", "module": "<.text.kernel>" }, { "id": 302, "name": "enqueue_hrtimer", "module": "<.text.kernel>" }, { "id": 303, "name": "timerqueue_add", "module": "<.text.kernel>" }, { "id": 304, "name": "arch_cpu_idle_enter", "module": "<.text.kernel>" }, { "id": 305, "name": "local_touch_nmi", "module": "<.text.kernel>" }, { "id": 306, "name": "atomic_notifier_call_chain", "module": "<.text.kernel>" }, { "id": 307, "name": "notifier_call_chain", "module": "<.text.kernel>" }, { "id": 308, "name": "tick_check_broadcast_expired", "module": "<.text.kernel>" }, { "id": 309, "name": "rcu_idle_enter", "module": "<.text.kernel>" }, { "id": 310, "name": "rcu_sysidle_enter", "module": "<.text.kernel>" }, { "id": 311, "name": "arch_cpu_idle", "module": "<.text.kernel>" }, { "id": 312, "name": "cpuidle_idle_call", "module": "<.text.kernel>" }, { "id": 313, "name": "cpuidle_get_cpu_driver", "module": "<.text.kernel>" }, { "id": 314, "name": "menu_select", "module": "<.text.kernel>" }, { "id": 315, "name": "pm_qos_request", "module": "<.text.kernel>" }, { "id": 316, "name": "tick_nohz_get_sleep_length", "module": "<.text.kernel>" }, { "id": 317, "name": "ns_to_timespec", "module": "<.text.kernel>" }, { "id": 318, "name": "nr_iowait_cpu", "module": "<.text.kernel>" }, { "id": 319, "name": "this_cpu_load", "module": "<.text.kernel>" }, { "id": 320, "name": "int_sqrt", "module": "<.text.kernel>" }, { "id": 321, "name": "menu_reflect", "module": "<.text.kernel>" }, { "id": 322, "name": "rcu_idle_exit", "module": "<.text.kernel>" }, { "id": 323, "name": "rcu_sysidle_exit", "module": "<.text.kernel>" }, { "id": 324, "name": "rcu_sysidle_force_exit", "module": "<.text.kernel>" }, { "id": 325, "name": "arch_cpu_idle_exit", "module": "<.text.kernel>" }, { "id": 326, "name": "tick_nohz_idle_exit", "module": "<.text.kernel>" }, { "id": 327, "name": "tick_nohz_stop_idle", "module": "<.text.kernel>" }, { "id": 328, "name": "update_ts_time_stats", "module": "<.text.kernel>" }, { "id": 329, "name": "sched_clock_idle_wakeup_event", "module": "<.text.kernel>" }, { "id": 330, "name": "sched_clock_tick", "module": "<.text.kernel>" }, { "id": 331, "name": "touch_softlockup_watchdog", "module": "<.text.kernel>" }, { "id": 332, "name": "tick_do_update_jiffies64", "module": "<.text.kernel>" }, { "id": 333, "name": "update_cpu_load_nohz", "module": "<.text.kernel>" }, { "id": 334, "name": "calc_load_exit_idle", "module": "<.text.kernel>" }, { "id": 335, "name": "tick_nohz_restart", "module": "<.text.kernel>" }, { "id": 336, "name": "hrtimer_cancel", "module": "<.text.kernel>" }, { "id": 337, "name": "hrtimer_try_to_cancel", "module": "<.text.kernel>" }, { "id": 338, "name": "hrtimer_forward", "module": "<.text.kernel>" }, { "id": 339, "name": "hrtimer_start_range_ns", "module": "<.text.kernel>" }, { "id": 340, "name": "pre_schedule_idle", "module": "<.text.kernel>" }, { "id": 341, "name": "idle_exit_fair", "module": "<.text.kernel>" }, { "id": 342, "name": "put_prev_task_idle", "module": "<.text.kernel>" }, { "id": -1, "name": "<Unknown>", "module": "<Unknown>" }],
        links: [{ "source": 36, "target": 37, "value": 88 }, { "source": 33, "target": 34, "value": 40 }, { "source": 32, "target": 33, "value": 118 }, { "source": 30, "target": 32, "value": 134 }, { "source": 29, "target": 30, "value": 244 }, { "source": 30, "target": 31, "value": 22 }, { "source": 15, "target": 16, "value": 234 }, { "source": 14, "target": 15, "value": 806 }, { "source": 13, "target": 14, "value": 814 }, { "source": 12, "target": 13, "value": 228 }, { "source": 11, "target": 12, "value": 308 }, { "source": 29, "target": 11, "value": 200 }, { "source": 52, "target": 53, "value": 60 }, { "source": 311, "target": 312, "value": 3472 }, { "source": 0, "target": 311, "value": 3492 }, { "source": 214, "target": 0, "value": 23990 }, { "source": 17, "target": 19, "value": 56 }, { "source": 9, "target": 17, "value": 492 }, { "source": 8, "target": 9, "value": 1324 }, { "source": 17, "target": 13, "value": 228 }, { "source": 17, "target": 18, "value": 88 }, { "source": 10, "target": 11, "value": 200 }, { "source": 9, "target": 10, "value": 228 }, { "source": 2, "target": 5, "value": 16 }, { "source": 2, "target": 4, "value": 52 }, { "source": 2, "target": 3, "value": 24 }, { "source": 282, "target": 16, "value": 108 }, { "source": 281, "target": 282, "value": 216 }, { "source": 314, "target": 320, "value": 1460 }, { "source": 312, "target": 314, "value": 3136 }, { "source": 314, "target": 318, "value": 36 }, { "source": 314, "target": 319, "value": 16 }, { "source": 314, "target": 317, "value": 50 }, { "source": 314, "target": 316, "value": 16 }, { "source": 314, "target": 315, "value": 18 }, { "source": 312, "target": 313, "value": 22 }, { "source": 309, "target": 310, "value": 48 }, { "source": 0, "target": 309, "value": 230 }, { "source": 309, "target": 36, "value": 136 }, { "source": 0, "target": 308, "value": 18 }, { "source": 306, "target": 307, "value": 24 }, { "source": 304, "target": 306, "value": 30 }, { "source": 0, "target": 304, "value": 64 }, { "source": 304, "target": 305, "value": 12 }, { "source": 294, "target": 120, "value": 68 }, { "source": 293, "target": 294, "value": 1448 }, { "source": 284, "target": 293, "value": 1466 }, { "source": 280, "target": 284, "value": 2154 }, { "source": 278, "target": 280, "value": 2502 }, { "source": 0, "target": 278, "value": 2564 }, { "source": 301, "target": 5, "value": 64 }, { "source": 300, "target": 301, "value": 304 }, { "source": 299, "target": 300, "value": 1056 }, { "source": 294, "target": 299, "value": 576 }, { "source": 301, "target": 16, "value": 72 }, { "source": 300, "target": 281, "value": 440 }, { "source": 303, "target": 158, "value": 40 }, { "source": 302, "target": 303, "value": 276 }, { "source": 294, "target": 302, "value": 380 }, { "source": 298, "target": 299, "value": 576 }, { "source": 296, "target": 298, "value": 792 }, { "source": 294, "target": 296, "value": 598 }, { "source": 297, "target": 191, "value": 80 }, { "source": 296, "target": 297, "value": 232 }, { "source": 297, "target": 190, "value": 64 }, { "source": 295, "target": 114, "value": 108 }, { "source": 294, "target": 295, "value": 160 }, { "source": 284, "target": 292, "value": 24 }, { "source": 284, "target": 291, "value": 36 }, { "source": 290, "target": 120, "value": 34 }, { "source": 289, "target": 290, "value": 132 }, { "source": 284, "target": 289, "value": 264 }, { "source": 290, "target": 114, "value": 36 }, { "source": 289, "target": 19, "value": 14 }, { "source": 289, "target": 18, "value": 22 }, { "source": 284, "target": 288, "value": 14 }, { "source": 286, "target": 287, "value": 66 }, { "source": 284, "target": 286, "value": 106 }, { "source": 284, "target": 285, "value": 24 }, { "source": 283, "target": 13, "value": 114 }, { "source": 280, "target": 283, "value": 126 }, { "source": 280, "target": 281, "value": 110 }, { "source": 278, "target": 279, "value": 24 }, { "source": 0, "target": 120, "value": 34 }, { "source": 274, "target": 275, "value": 80 }, { "source": 0, "target": 274, "value": 92 }, { "source": 0, "target": 114, "value": 36 }, { "source": 0, "target": 8, "value": 1516 }, { "source": 213, "target": 19, "value": 14 }, { "source": 9, "target": 213, "value": 548 }, { "source": 213, "target": 29, "value": 474 }, { "source": 213, "target": 18, "value": 22 }, { "source": 0, "target": 2, "value": 626 }, { "source": 0, "target": 1, "value": 36 }, { "source": 0, "target": 212, "value": 56 }, { "source": 170, "target": 171, "value": 64 }, { "source": 169, "target": 170, "value": 128 }, { "source": 211, "target": 169, "value": 338 }, { "source": 0, "target": 211, "value": 426 }, { "source": 207, "target": 210, "value": 12 }, { "source": 197, "target": 207, "value": 242 }, { "source": 192, "target": 197, "value": 998 }, { "source": 0, "target": 192, "value": 2422 }, { "source": 208, "target": 209, "value": 100 }, { "source": 207, "target": 208, "value": 388 }, { "source": 197, "target": 206, "value": 8 }, { "source": 203, "target": 204, "value": 144 }, { "source": 197, "target": 203, "value": 46 }, { "source": 199, "target": 205, "value": 152 }, { "source": 198, "target": 199, "value": 1092 }, { "source": 197, "target": 198, "value": 574 }, { "source": 199, "target": 203, "value": 92 }, { "source": 202, "target": 14, "value": 148 }, { "source": 201, "target": 202, "value": 168 }, { "source": 200, "target": 201, "value": 576 }, { "source": 199, "target": 200, "value": 724 }, { "source": 192, "target": 193, "value": 88 }, { "source": 0, "target": 273, "value": 16 }, { "source": 0, "target": 186, "value": 712 }, { "source": 0, "target": 272, "value": 46 }, { "source": 0, "target": 271, "value": 24 }, { "source": 0, "target": 183, "value": 126 }, { "source": 261, "target": 18, "value": 22 }, { "source": 0, "target": 261, "value": 5970 }, { "source": 261, "target": 267, "value": 44 }, { "source": 261, "target": 13, "value": 456 }, { "source": 261, "target": 264, "value": 4154 }, { "source": 269, "target": 270, "value": 960 }, { "source": 264, "target": 269, "value": 352 }, { "source": 265, "target": 142, "value": 66 }, { "source": 264, "target": 265, "value": 3250 }, { "source": 265, "target": 145, "value": 172 }, { "source": 265, "target": 143, "value": 66 }, { "source": 265, "target": 269, "value": 1208 }, { "source": 262, "target": 120, "value": 34 }, { "source": 261, "target": 262, "value": 1022 }, { "source": 262, "target": 153, "value": 218 }, { "source": 149, "target": 13, "value": 228 }, { "source": 263, "target": 149, "value": 142 }, { "source": 262, "target": 263, "value": 160 }, { "source": 262, "target": 114, "value": 36 }, { "source": 261, "target": 19, "value": 14 }, { "source": 221, "target": 159, "value": 8 }, { "source": 220, "target": 221, "value": 1544 }, { "source": 219, "target": 220, "value": 1752 }, { "source": 0, "target": 219, "value": 1776 }, { "source": 222, "target": 155, "value": 438 }, { "source": 221, "target": 222, "value": 1348 }, { "source": 222, "target": 184, "value": 76 }, { "source": 222, "target": 223, "value": 102 }, { "source": 222, "target": 187, "value": 48 }, { "source": 222, "target": 153, "value": 68 }, { "source": 152, "target": 185, "value": 32 }, { "source": 222, "target": 152, "value": 234 }, { "source": 152, "target": 184, "value": 72 }, { "source": 220, "target": 149, "value": 142 }, { "source": 0, "target": 182, "value": 52 }, { "source": 0, "target": 181, "value": 76 }, { "source": 152, "target": 260, "value": 92 }, { "source": 155, "target": 154, "value": 34 }, { "source": 155, "target": 223, "value": 34 }, { "source": 155, "target": 152, "value": 226 }, { "source": 276, "target": 0, "value": 3532 }, { "source": 0, "target": 276, "value": 3548 }, { "source": 224, "target": 207, "value": 242 }, { "source": 192, "target": 224, "value": 998 }, { "source": 224, "target": 206, "value": 8 }, { "source": 224, "target": 203, "value": 46 }, { "source": 224, "target": 198, "value": 574 }, { "source": 192, "target": 195, "value": 88 }, { "source": 188, "target": 191, "value": 84 }, { "source": 186, "target": 188, "value": 428 }, { "source": 188, "target": 190, "value": 56 }, { "source": 188, "target": 189, "value": 132 }, { "source": 186, "target": 187, "value": 48 }, { "source": 0, "target": 342, "value": 10 }, { "source": 340, "target": 341, "value": 76 }, { "source": 0, "target": 340, "value": 98 }, { "source": 339, "target": 294, "value": 822 }, { "source": 335, "target": 339, "value": 836 }, { "source": 326, "target": 335, "value": 1738 }, { "source": 0, "target": 326, "value": 2184 }, { "source": 335, "target": 338, "value": 20 }, { "source": 337, "target": 120, "value": 34 }, { "source": 336, "target": 337, "value": 796 }, { "source": 335, "target": 336, "value": 822 }, { "source": 337, "target": 296, "value": 598 }, { "source": 337, "target": 295, "value": 80 }, { "source": 326, "target": 331, "value": 12 }, { "source": 326, "target": 334, "value": 20 }, { "source": 326, "target": 333, "value": 28 }, { "source": 326, "target": 332, "value": 24 }, { "source": 329, "target": 331, "value": 12 }, { "source": 327, "target": 329, "value": 38 }, { "source": 326, "target": 327, "value": 148 }, { "source": 329, "target": 330, "value": 8 }, { "source": 328, "target": 318, "value": 18 }, { "source": 327, "target": 328, "value": 76 }, { "source": 326, "target": 281, "value": 110 }, { "source": 325, "target": 306, "value": 30 }, { "source": 0, "target": 325, "value": 56 }, { "source": 174, "target": 176, "value": 60 }, { "source": 173, "target": 174, "value": 94 }, { "source": 0, "target": 322, "value": 222 }, { "source": 174, "target": 175, "value": 12 }, { "source": 323, "target": 324, "value": 18 }, { "source": 322, "target": 323, "value": 66 }, { "source": 322, "target": 52, "value": 110 }, { "source": 312, "target": 321, "value": 22 }, { "source": 312, "target": 173, "value": 184 }, { "source": 266, "target": 268, "value": 30 }, { "source": 265, "target": 266, "value": 230 }, { "source": 266, "target": 267, "value": 22 }],
    },
    launchthreads: {
        functions: [{ "id": 0, "name": "native_write_msr_safe", "module": "<.text.kernel>" }, { "id": 4, "name": "mutex_unlock", "module": "<.text.kernel>" }, { "id": 5, "name": "free_pages", "module": "<.text.kernel>" }, { "id": 6, "name": "free_pages.part.67", "module": "<.text.kernel>" }, { "id": 7, "name": "__free_pages", "module": "<.text.kernel>" }, { "id": 8, "name": "free_hot_cold_page", "module": "<.text.kernel>" }, { "id": 9, "name": "free_pages_prepare", "module": "<.text.kernel>" }, { "id": 10, "name": "get_pageblock_flags_group", "module": "<.text.kernel>" }, { "id": 14, "name": "acct_update_integrals", "module": "<.text.kernel>" }, { "id": 15, "name": "task_cputime", "module": "<.text.kernel>" }, { "id": 16, "name": "fetch_task_cputime", "module": "<.text.kernel>" }, { "id": 17, "name": "local_clock", "module": "<.text.kernel>" }, { "id": 18, "name": "sched_clock_cpu", "module": "<.text.kernel>" }, { "id": 19, "name": "sched_clock", "module": "<.text.kernel>" }, { "id": 20, "name": "native_sched_clock", "module": "<.text.kernel>" }, { "id": 21, "name": "native_read_tsc", "module": "<.text.kernel>" }, { "id": 22, "name": "__acct_update_integrals", "module": "<.text.kernel>" }, { "id": 23, "name": "jiffies_to_timeval", "module": "<.text.kernel>" }, { "id": 24, "name": "task_numa_free", "module": "<.text.kernel>" }, { "id": 25, "name": "kfree", "module": "<.text.kernel>" }, { "id": 27, "name": "__slab_free", "module": "<.text.kernel>" }, { "id": 31, "name": "kmem_cache_free", "module": "<.text.kernel>" }, { "id": 38, "name": "syscall_trace_leave", "module": "<.text.kernel>" }, { "id": 39, "name": "context_tracking_user_exit", "module": "<.text.kernel>" }, { "id": 40, "name": "context_tracking_user_enter", "module": "<.text.kernel>" }, { "id": 41, "name": "vtime_user_enter", "module": "<.text.kernel>" }, { "id": 42, "name": "_raw_spin_lock", "module": "<.text.kernel>" }, { "id": 43, "name": "__vtime_account_system", "module": "<.text.kernel>" }, { "id": 44, "name": "get_vtime_delta", "module": "<.text.kernel>" }, { "id": 45, "name": "account_system_time", "module": "<.text.kernel>" }, { "id": 46, "name": "cpuacct_account_field", "module": "<.text.kernel>" }, { "id": 47, "name": "acct_account_cputime", "module": "<.text.kernel>" }, { "id": 48, "name": "_raw_spin_unlock", "module": "<.text.kernel>" }, { "id": 49, "name": "rcu_user_enter", "module": "<.text.kernel>" }, { "id": 50, "name": "rcu_eqs_enter", "module": "<.text.kernel>" }, { "id": 51, "name": "rcu_eqs_enter_common.isra.47", "module": "<.text.kernel>" }, { "id": 56, "name": "page_fault", "module": "<.text.kernel>" }, { "id": 58, "name": "error_entry", "module": "<.text.kernel>" }, { "id": 61, "name": "do_page_fault", "module": "<.text.kernel>" }, { "id": 62, "name": "rcu_user_exit", "module": "<.text.kernel>" }, { "id": 63, "name": "rcu_eqs_exit", "module": "<.text.kernel>" }, { "id": 64, "name": "rcu_eqs_exit_common.isra.48", "module": "<.text.kernel>" }, { "id": 65, "name": "vtime_account_user", "module": "<.text.kernel>" }, { "id": 66, "name": "account_user_time", "module": "<.text.kernel>" }, { "id": 67, "name": "__do_page_fault", "module": "<.text.kernel>" }, { "id": 68, "name": "down_read_trylock", "module": "<.text.kernel>" }, { "id": 69, "name": "_cond_resched", "module": "<.text.kernel>" }, { "id": 70, "name": "find_vma", "module": "<.text.kernel>" }, { "id": 71, "name": "handle_mm_fault", "module": "<.text.kernel>" }, { "id": 72, "name": "__mem_cgroup_count_vm_event", "module": "<.text.kernel>" }, { "id": 73, "name": "is_vma_temporary_stack", "module": "<.text.kernel>" }, { "id": 74, "name": "__pte_alloc", "module": "<.text.kernel>" }, { "id": 75, "name": "pte_alloc_one", "module": "<.text.kernel>" }, { "id": 76, "name": "alloc_pages_current", "module": "<.text.kernel>" }, { "id": 77, "name": "policy_nodemask", "module": "<.text.kernel>" }, { "id": 78, "name": "policy_zonelist", "module": "<.text.kernel>" }, { "id": 79, "name": "__alloc_pages_nodemask", "module": "<.text.kernel>" }, { "id": 80, "name": "next_zones_zonelist", "module": "<.text.kernel>" }, { "id": 81, "name": "get_page_from_freelist", "module": "<.text.kernel>" }, { "id": 82, "name": "__zone_watermark_ok", "module": "<.text.kernel>" }, { "id": 83, "name": "__mod_zone_page_state", "module": "<.text.kernel>" }, { "id": 84, "name": "zone_statistics", "module": "<.text.kernel>" }, { "id": 85, "name": "__inc_zone_state", "module": "<.text.kernel>" }, { "id": 86, "name": "clear_page", "module": "<.text.kernel>" }, { "id": 88, "name": "inc_zone_page_state", "module": "<.text.kernel>" }, { "id": 89, "name": "native_set_pmd", "module": "<.text.kernel>" }, { "id": 90, "name": "__do_fault", "module": "<.text.kernel>" }, { "id": 91, "name": "filemap_fault", "module": "<.text.kernel>" }, { "id": 92, "name": "find_get_page", "module": "<.text.kernel>" }, { "id": 93, "name": "radix_tree_lookup_slot", "module": "<.text.kernel>" }, { "id": 94, "name": "radix_tree_lookup_element", "module": "<.text.kernel>" }, { "id": 95, "name": "add_mm_counter_fast", "module": "<.text.kernel>" }, { "id": 96, "name": "page_add_file_rmap", "module": "<.text.kernel>" }, { "id": 97, "name": "native_set_pte_at", "module": "<.text.kernel>" }, { "id": 98, "name": "unlock_page", "module": "<.text.kernel>" }, { "id": 99, "name": "page_waitqueue", "module": "<.text.kernel>" }, { "id": 100, "name": "__wake_up_bit", "module": "<.text.kernel>" }, { "id": 101, "name": "up_read", "module": "<.text.kernel>" }, { "id": 106, "name": "lookup_page_cgroup", "module": "<.text.kernel>" }, { "id": 109, "name": "anon_vma_prepare", "module": "<.text.kernel>" }, { "id": 110, "name": "alloc_pages_vma", "module": "<.text.kernel>" }, { "id": 111, "name": "get_vma_policy", "module": "<.text.kernel>" }, { "id": 114, "name": "mem_cgroup_newpage_charge", "module": "<.text.kernel>" }, { "id": 115, "name": "mem_cgroup_charge_common", "module": "<.text.kernel>" }, { "id": 116, "name": "__mem_cgroup_try_charge", "module": "<.text.kernel>" }, { "id": 117, "name": "__mem_cgroup_commit_charge", "module": "<.text.kernel>" }, { "id": 118, "name": "mem_cgroup_charge_statistics.isra.31", "module": "<.text.kernel>" }, { "id": 119, "name": "memcg_check_events", "module": "<.text.kernel>" }, { "id": 123, "name": "find_next_bit", "module": "<.text.kernel>" }, { "id": 124, "name": "page_add_new_anon_rmap", "module": "<.text.kernel>" }, { "id": 125, "name": "lru_cache_add", "module": "<.text.kernel>" }, { "id": 126, "name": "__lru_cache_add", "module": "<.text.kernel>" }, { "id": 131, "name": "put_page", "module": "<.text.kernel>" }, { "id": 141, "name": "kmem_cache_alloc", "module": "<.text.kernel>" }, { "id": 142, "name": "find_mergeable_anon_vma", "module": "<.text.kernel>" }, { "id": 143, "name": "reusable_anon_vma", "module": "<.text.kernel>" }, { "id": 145, "name": "down_write", "module": "<.text.kernel>" }, { "id": 146, "name": "anon_vma_interval_tree_insert", "module": "<.text.kernel>" }, { "id": 147, "name": "__rb_insert_augmented", "module": "<.text.kernel>" }, { "id": 148, "name": "up_write", "module": "<.text.kernel>" }, { "id": 160, "name": "system_call", "module": "<.text.kernel>" }, { "id": 163, "name": "syscall_trace_enter", "module": "<.text.kernel>" }, { "id": 174, "name": "ld-2.19.so@ab0", "module": "ld-2.19.so" }, { "id": 197, "name": "ld-2.19.so@19c70", "module": "ld-2.19.so" }, { "id": 232, "name": "memset", "module": "<.text.kernel>" }, { "id": 255, "name": "call_rcu_sched", "module": "<.text.kernel>" }, { "id": 256, "name": "__call_rcu", "module": "<.text.kernel>" }, { "id": 257, "name": "__call_rcu_nocb_enqueue", "module": "<.text.kernel>" }, { "id": 258, "name": "__wake_up", "module": "<.text.kernel>" }, { "id": 259, "name": "_raw_spin_lock_irqsave", "module": "<.text.kernel>" }, { "id": 260, "name": "__wake_up_common", "module": "<.text.kernel>" }, { "id": 261, "name": "autoremove_wake_function", "module": "<.text.kernel>" }, { "id": 262, "name": "default_wake_function", "module": "<.text.kernel>" }, { "id": 263, "name": "try_to_wake_up", "module": "<.text.kernel>" }, { "id": 264, "name": "task_waking_fair", "module": "<.text.kernel>" }, { "id": 265, "name": "select_task_rq_fair", "module": "<.text.kernel>" }, { "id": 266, "name": "source_load", "module": "<.text.kernel>" }, { "id": 267, "name": "target_load", "module": "<.text.kernel>" }, { "id": 268, "name": "idle_cpu", "module": "<.text.kernel>" }, { "id": 269, "name": "cpus_share_cache", "module": "<.text.kernel>" }, { "id": 270, "name": "ttwu_do_activate.constprop.74", "module": "<.text.kernel>" }, { "id": 271, "name": "activate_task", "module": "<.text.kernel>" }, { "id": 272, "name": "enqueue_task", "module": "<.text.kernel>" }, { "id": 273, "name": "update_rq_clock.part.63", "module": "<.text.kernel>" }, { "id": 274, "name": "enqueue_task_fair", "module": "<.text.kernel>" }, { "id": 275, "name": "enqueue_entity", "module": "<.text.kernel>" }, { "id": 276, "name": "update_curr", "module": "<.text.kernel>" }, { "id": 277, "name": "update_cfs_rq_blocked_load", "module": "<.text.kernel>" }, { "id": 278, "name": "account_entity_enqueue", "module": "<.text.kernel>" }, { "id": 279, "name": "update_cfs_shares", "module": "<.text.kernel>" }, { "id": 280, "name": "place_entity", "module": "<.text.kernel>" }, { "id": 281, "name": "__enqueue_entity", "module": "<.text.kernel>" }, { "id": 282, "name": "rb_insert_color", "module": "<.text.kernel>" }, { "id": 283, "name": "hrtick_update", "module": "<.text.kernel>" }, { "id": 284, "name": "ttwu_do_wakeup", "module": "<.text.kernel>" }, { "id": 285, "name": "check_preempt_curr", "module": "<.text.kernel>" }, { "id": 286, "name": "resched_task", "module": "<.text.kernel>" }, { "id": 287, "name": "probe_sched_wakeup", "module": ".text.simple_pt" }, { "id": 288, "name": "probe_sched_wakeup_pid.isra.10", "module": ".text.simple_pt" }, { "id": 290, "name": "probe_sched_wakeup", "module": "<.text.kernel>" }, { "id": 291, "name": "tracing_record_cmdline", "module": "<.text.kernel>" }, { "id": 292, "name": "tracing_is_on", "module": "<.text.kernel>" }, { "id": 293, "name": "ring_buffer_record_is_on", "module": "<.text.kernel>" }, { "id": 294, "name": "ttwu_stat", "module": "<.text.kernel>" }, { "id": 295, "name": "_raw_spin_unlock_irqrestore", "module": "<.text.kernel>" }, { "id": 301, "name": "ld-2.19.so@1a9c0", "module": "ld-2.19.so" }, { "id": 305, "name": "SyS_mmap", "module": "<.text.kernel>" }, { "id": 306, "name": "SyS_mmap_pgoff", "module": "<.text.kernel>" }, { "id": 307, "name": "vm_mmap_pgoff", "module": "<.text.kernel>" }, { "id": 308, "name": "security_mmap_file", "module": "<.text.kernel>" }, { "id": 309, "name": "apparmor_mmap_file", "module": "<.text.kernel>" }, { "id": 310, "name": "common_mmap", "module": "<.text.kernel>" }, { "id": 311, "name": "ima_file_mmap", "module": "<.text.kernel>" }, { "id": 312, "name": "do_mmap_pgoff", "module": "<.text.kernel>" }, { "id": 313, "name": "get_unmapped_area", "module": "<.text.kernel>" }, { "id": 314, "name": "arch_get_unmapped_area_topdown", "module": "<.text.kernel>" }, { "id": 315, "name": "unmapped_area_topdown", "module": "<.text.kernel>" }, { "id": 316, "name": "security_mmap_addr", "module": "<.text.kernel>" }, { "id": 317, "name": "cap_mmap_addr", "module": "<.text.kernel>" }, { "id": 318, "name": "mmap_region", "module": "<.text.kernel>" }, { "id": 319, "name": "ftrace_regs_caller", "module": "<.text.kernel>" }, { "id": 321, "name": "ftrace_ops_list_func", "module": "<.text.kernel>" }, { "id": 322, "name": "ftrace_ops_test", "module": "<.text.kernel>" }, { "id": 323, "name": "ftrace_lookup_ip", "module": "<.text.kernel>" }, { "id": 324, "name": "kprobe_ftrace_handler", "module": "<.text.kernel>" }, { "id": 325, "name": "__fentry__", "module": "<.text.kernel>" }, { "id": 326, "name": "get_kprobe", "module": "<.text.kernel>" }, { "id": 327, "name": "probe_mmap_region", "module": ".text.simple_pt" }, { "id": 330, "name": "security_vm_enough_memory_mm", "module": "<.text.kernel>" }, { "id": 331, "name": "cap_vm_enough_memory", "module": "<.text.kernel>" }, { "id": 332, "name": "cap_capable", "module": "<.text.kernel>" }, { "id": 333, "name": "__vm_enough_memory", "module": "<.text.kernel>" }, { "id": 334, "name": "__percpu_counter_add", "module": "<.text.kernel>" }, { "id": 335, "name": "vma_merge", "module": "<.text.kernel>" }, { "id": 336, "name": "can_vma_merge_before", "module": "<.text.kernel>" }, { "id": 337, "name": "vma_wants_writenotify", "module": "<.text.kernel>" }, { "id": 338, "name": "vma_link", "module": "<.text.kernel>" }, { "id": 339, "name": "__vma_link_list", "module": "<.text.kernel>" }, { "id": 340, "name": "__vma_link_rb", "module": "<.text.kernel>" }, { "id": 341, "name": "vma_compute_subtree_gap", "module": "<.text.kernel>" }, { "id": 342, "name": "__vma_link_file", "module": "<.text.kernel>" }, { "id": 343, "name": "perf_event_mmap", "module": "<.text.kernel>" }, { "id": 344, "name": "vm_stat_account", "module": "<.text.kernel>" }, { "id": 345, "name": "ld-2.19.so@1a9b0", "module": "ld-2.19.so" }, { "id": 366, "name": "ld-2.19.so@111c0", "module": "ld-2.19.so" }, { "id": 402, "name": "try_module_get", "module": "<.text.kernel>" }, { "id": 417, "name": "mutex_lock", "module": "<.text.kernel>" }, { "id": 453, "name": "__slab_alloc", "module": "<.text.kernel>" }, { "id": 454, "name": "arch_local_irq_save", "module": "<.text.kernel>" }, { "id": 455, "name": "__cmpxchg_double_slab.isra.44", "module": "<.text.kernel>" }, { "id": 460, "name": "vma_gap_callbacks_rotate", "module": "<.text.kernel>" }, { "id": 487, "name": "__d_lookup", "module": "<.text.kernel>" }, { "id": 530, "name": "trace_event_buffer_lock_reserve", "module": "<.text.kernel>" }, { "id": 531, "name": "trace_buffer_lock_reserve", "module": "<.text.kernel>" }, { "id": 532, "name": "ring_buffer_lock_reserve", "module": "<.text.kernel>" }, { "id": 533, "name": "rb_reserve_next_event", "module": "<.text.kernel>" }, { "id": 534, "name": "trace_clock_local", "module": "<.text.kernel>" }, { "id": 535, "name": "ring_buffer_event_data", "module": "<.text.kernel>" }, { "id": 536, "name": "rb_event_data", "module": "<.text.kernel>" }, { "id": 537, "name": "tracing_generic_entry_update", "module": "<.text.kernel>" }, { "id": 539, "name": "filter_check_discard", "module": "<.text.kernel>" }, { "id": 540, "name": "trace_buffer_unlock_commit", "module": "<.text.kernel>" }, { "id": 541, "name": "ring_buffer_unlock_commit", "module": "<.text.kernel>" }, { "id": 542, "name": "rb_update_write_stamp", "module": "<.text.kernel>" }, { "id": 543, "name": "ftrace_trace_userstack", "module": "<.text.kernel>" }, { "id": 546, "name": "SyS_mprotect", "module": "<.text.kernel>" }, { "id": 547, "name": "security_file_mprotect", "module": "<.text.kernel>" }, { "id": 548, "name": "apparmor_file_mprotect", "module": "<.text.kernel>" }, { "id": 549, "name": "mprotect_fixup", "module": "<.text.kernel>" }, { "id": 550, "name": "split_vma", "module": "<.text.kernel>" }, { "id": 551, "name": "__split_vma.isra.34", "module": "<.text.kernel>" }, { "id": 552, "name": "vma_dup_policy", "module": "<.text.kernel>" }, { "id": 553, "name": "anon_vma_clone", "module": "<.text.kernel>" }, { "id": 554, "name": "vma_adjust", "module": "<.text.kernel>" }, { "id": 557, "name": "vm_get_page_prot", "module": "<.text.kernel>" }, { "id": 558, "name": "change_protection", "module": "<.text.kernel>" }, { "id": 559, "name": "change_protection_range", "module": "<.text.kernel>" }, { "id": 565, "name": "lru_add_drain", "module": "<.text.kernel>" }, { "id": 566, "name": "lru_add_drain_cpu", "module": "<.text.kernel>" }, { "id": 567, "name": "pagevec_lru_move_fn", "module": "<.text.kernel>" }, { "id": 568, "name": "mem_cgroup_page_lruvec", "module": "<.text.kernel>" }, { "id": 569, "name": "__pagevec_lru_add_fn", "module": "<.text.kernel>" }, { "id": 570, "name": "mem_cgroup_update_lru_size", "module": "<.text.kernel>" }, { "id": 571, "name": "release_pages", "module": "<.text.kernel>" }, { "id": 572, "name": "free_hot_cold_page_list", "module": "<.text.kernel>" }, { "id": 575, "name": "unmap_single_vma", "module": "<.text.kernel>" }, { "id": 576, "name": "unmap_page_range", "module": "<.text.kernel>" }, { "id": 577, "name": "mem_cgroup_uncharge_start", "module": "<.text.kernel>" }, { "id": 578, "name": "sync_mm_rss", "module": "<.text.kernel>" }, { "id": 579, "name": "mem_cgroup_uncharge_end", "module": "<.text.kernel>" }, { "id": 585, "name": "tlb_finish_mmu", "module": "<.text.kernel>" }, { "id": 598, "name": "native_apic_mem_write", "module": "<.text.kernel>" }, { "id": 601, "name": "rcu_sysidle_exit", "module": "<.text.kernel>" }, { "id": 603, "name": "vtime_account_system", "module": "<.text.kernel>" }, { "id": 608, "name": "read_tsc", "module": "<.text.kernel>" }, { "id": 610, "name": "__remove_hrtimer", "module": "<.text.kernel>" }, { "id": 611, "name": "timerqueue_del", "module": "<.text.kernel>" }, { "id": 612, "name": "rb_next", "module": "<.text.kernel>" }, { "id": 613, "name": "rb_erase", "module": "<.text.kernel>" }, { "id": 615, "name": "ktime_get", "module": "<.text.kernel>" }, { "id": 617, "name": "tick_do_update_jiffies64", "module": "<.text.kernel>" }, { "id": 623, "name": "set_normalized_timespec", "module": "<.text.kernel>" }, { "id": 625, "name": "notifier_call_chain", "module": "<.text.kernel>" }, { "id": 638, "name": "sched_clock_tick", "module": "<.text.kernel>" }, { "id": 640, "name": "update_min_vruntime", "module": "<.text.kernel>" }, { "id": 641, "name": "cpuacct_charge", "module": "<.text.kernel>" }, { "id": 642, "name": "__calc_delta", "module": "<.text.kernel>" }, { "id": 643, "name": "account_entity_dequeue", "module": "<.text.kernel>" }, { "id": 653, "name": "hrtimer_forward", "module": "<.text.kernel>" }, { "id": 654, "name": "enqueue_hrtimer", "module": "<.text.kernel>" }, { "id": 655, "name": "timerqueue_add", "module": "<.text.kernel>" }, { "id": 656, "name": "tick_program_event", "module": "<.text.kernel>" }, { "id": 657, "name": "clockevents_program_event", "module": "<.text.kernel>" }, { "id": 658, "name": "lapic_next_deadline", "module": "<.text.kernel>" }, { "id": 662, "name": "msecs_to_jiffies", "module": "<.text.kernel>" }, { "id": 665, "name": "_raw_spin_lock_irq", "module": "<.text.kernel>" }, { "id": 668, "name": "wake_up_process", "module": "<.text.kernel>" }, { "id": 669, "name": "check_preempt_wakeup", "module": "<.text.kernel>" }, { "id": 670, "name": "wakeup_gran.isra.47", "module": "<.text.kernel>" }, { "id": 671, "name": "set_next_buddy", "module": "<.text.kernel>" }, { "id": 675, "name": "update_blocked_averages", "module": "<.text.kernel>" }, { "id": 676, "name": "update_rq_clock", "module": "<.text.kernel>" }, { "id": 679, "name": "rcu_sysidle_enter", "module": "<.text.kernel>" }, { "id": 682, "name": "schedule_user", "module": "<.text.kernel>" }, { "id": 683, "name": "__schedule", "module": "<.text.kernel>" }, { "id": 684, "name": "rcu_note_context_switch", "module": "<.text.kernel>" }, { "id": 685, "name": "put_prev_task_fair", "module": "<.text.kernel>" }, { "id": 686, "name": "pick_next_task_fair", "module": "<.text.kernel>" }, { "id": 687, "name": "clear_buddies", "module": "<.text.kernel>" }, { "id": 688, "name": "set_next_entity", "module": "<.text.kernel>" }, { "id": 689, "name": "update_stats_wait_end", "module": "<.text.kernel>" }, { "id": 690, "name": "probe_sched_switch", "module": ".text.simple_pt" }, { "id": 691, "name": "poor_mans_ptwrite_prev_pid", "module": ".text.simple_pt" }, { "id": 693, "name": "poor_mans_ptwrite_pid", "module": ".text.simple_pt" }, { "id": 695, "name": "probe_sched_switch", "module": "<.text.kernel>" }, { "id": 696, "name": "perf_event_task_sched_out", "module": "<.text.kernel>" }, { "id": 697, "name": "__context_tracking_task_switch", "module": "<.text.kernel>" }, { "id": 698, "name": "__switch_to", "module": "<.text.kernel>" }, { "id": 699, "name": "native_load_sp0", "module": "<.text.kernel>" }, { "id": 700, "name": "native_load_tls", "module": "<.text.kernel>" }, { "id": 701, "name": "finish_task_switch", "module": "<.text.kernel>" }, { "id": 702, "name": "vtime_common_task_switch", "module": "<.text.kernel>" }, { "id": 703, "name": "arch_vtime_task_switch", "module": "<.text.kernel>" }, { "id": 704, "name": "schedule", "module": "<.text.kernel>" }, { "id": 711, "name": "prepare_to_wait_event", "module": "<.text.kernel>" }, { "id": 712, "name": "finish_wait", "module": "<.text.kernel>" }, { "id": 718, "name": "effective_load.isra.35", "module": "<.text.kernel>" }, { "id": 723, "name": "deactivate_task", "module": "<.text.kernel>" }, { "id": 724, "name": "dequeue_task", "module": "<.text.kernel>" }, { "id": 725, "name": "dequeue_task_fair", "module": "<.text.kernel>" }, { "id": 726, "name": "dequeue_entity", "module": "<.text.kernel>" }, { "id": 727, "name": "ftrace_raw_event_switch_in", "module": ".text.simple_pt" }, { "id": 735, "name": "__vma_adjust_trans_huge", "module": "<.text.kernel>" }, { "id": 736, "name": "anon_vma_interval_tree_remove", "module": "<.text.kernel>" }, { "id": 773, "name": "do_arch_prctl", "module": "<.text.kernel>" }, { "id": 787, "name": "ld-2.19.so@a860", "module": "ld-2.19.so" }, { "id": 788, "name": "ld-2.19.so@9ef0", "module": "ld-2.19.so" }, { "id": 791, "name": "ld-2.19.so@9d70", "module": "ld-2.19.so" }, { "id": 818, "name": "libc-2.19.so@8c530", "module": "libc-2.19.so" }, { "id": 832, "name": "ld-2.19.so@f460", "module": "ld-2.19.so" }, { "id": 878, "name": "blocking_notifier_call_chain", "module": "<.text.kernel>" }, { "id": 879, "name": "__blocking_notifier_call_chain", "module": "<.text.kernel>" }, { "id": 887, "name": "ftrace_raw_event_switch_out", "module": ".text.simple_pt" }, { "id": 894, "name": "task_work_run", "module": "<.text.kernel>" }, { "id": 905, "name": "module_put", "module": "<.text.kernel>" }, { "id": 913, "name": "__task_pid_nr_ns", "module": "<.text.kernel>" }, { "id": 914, "name": "SyS_set_robust_list", "module": "<.text.kernel>" }, { "id": 915, "name": "SyS_futex", "module": "<.text.kernel>" }, { "id": 916, "name": "do_futex", "module": "<.text.kernel>" }, { "id": 917, "name": "futex_wait", "module": "<.text.kernel>" }, { "id": 918, "name": "futex_wait_setup", "module": "<.text.kernel>" }, { "id": 919, "name": "get_futex_key", "module": "<.text.kernel>" }, { "id": 920, "name": "get_futex_key_refs.isra.13", "module": "<.text.kernel>" }, { "id": 921, "name": "get_futex_value_locked", "module": "<.text.kernel>" }, { "id": 922, "name": "drop_futex_key_refs.isra.14", "module": "<.text.kernel>" }, { "id": 931, "name": "recalc_sigpending", "module": "<.text.kernel>" }, { "id": 932, "name": "recalc_sigpending_tsk", "module": "<.text.kernel>" }, { "id": 1027, "name": "main@400a10", "module": "main" }, { "id": 1038, "name": "main", "module": "main" }, { "id": 1039, "name": "rdtsc()", "module": "main" }, { "id": 1040, "name": "main@4009b0", "module": "main" }, { "id": 1043, "name": "_IO_vfprintf", "module": "libc-2.19.so" }, { "id": 1044, "name": "strchrnul", "module": "libc-2.19.so" }, { "id": 1045, "name": "_pthread_cleanup_push_defer", "module": "libpthread-2.19.so" }, { "id": 1046, "name": "_IO_file_xsputn", "module": "libc-2.19.so" }, { "id": 1059, "name": "_IO_setb", "module": "libc-2.19.so" }, { "id": 1061, "name": "_IO_default_xsputn", "module": "libc-2.19.so" }, { "id": 1062, "name": "__rmqueue", "module": "<.text.kernel>" }, { "id": 1063, "name": "libc-2.19.so@1f66b", "module": "libc-2.19.so" }, { "id": 1065, "name": "libc-2.19.so@49990", "module": "libc-2.19.so" }, { "id": 1067, "name": "_IO_padn", "module": "libc-2.19.so" }, { "id": 1068, "name": "libc-2.19.so@8c710", "module": "libc-2.19.so" }, { "id": 1069, "name": "__pthread_cleanup_pop_restore", "module": "libpthread-2.19.so" }, { "id": 1070, "name": "gettid()", "module": "main" }, { "id": 1071, "name": "main@400a50", "module": "main" }, { "id": 1074, "name": "sys_gettid", "module": "<.text.kernel>" }, { "id": 1075, "name": "main@400a20", "module": "main" }, { "id": 1084, "name": "thread_work()", "module": "main" }, { "id": 1085, "name": "f1()", "module": "main" }, { "id": 1086, "name": "f2()", "module": "main" }, { "id": 1087, "name": "f3()", "module": "main" }, { "id": 1088, "name": "f4()", "module": "main" }, { "id": 1089, "name": "f5()", "module": "main" }, { "id": 1090, "name": "f6()", "module": "main" }, { "id": 1091, "name": "f7()", "module": "main" }, { "id": 1092, "name": "f8()", "module": "main" }, { "id": 1093, "name": "fibonacci(int)", "module": "main" }, { "id": 1094, "name": "factorial(int)", "module": "main" }, { "id": 1095, "name": "is_prime(int)", "module": "main" }, { "id": 1096, "name": "main@400ac0", "module": "main" }, { "id": 1099, "name": "libstdc++.so.6.0.19@59590", "module": "libstdc++.so.6.0.19" }, { "id": 1116, "name": "libc-2.19.so@7fc80", "module": "libc-2.19.so" }, { "id": 1135, "name": "do_huge_pmd_anonymous_page", "module": "<.text.kernel>" }, { "id": 1137, "name": "main@4009e0", "module": "main" }, { "id": 1140, "name": "libstdc++.so.6.0.19@591b0", "module": "libstdc++.so.6.0.19" }, { "id": 1142, "name": "main@400a40", "module": "main" }, { "id": 1145, "name": "vsprintf", "module": "libc-2.19.so" }, { "id": 1146, "name": "libc-2.19.so@7bce0", "module": "libc-2.19.so" }, { "id": 1147, "name": "libc-2.19.so@7bc30", "module": "libc-2.19.so" }, { "id": 1149, "name": "libc-2.19.so@7d0a0", "module": "libc-2.19.so" }, { "id": 1151, "name": "main@400a30", "module": "main" }, { "id": 1154, "name": "libpthread-2.19.so@57d0", "module": "libpthread-2.19.so" }, { "id": 1157, "name": "libpthread-2.19.so@5640", "module": "libpthread-2.19.so" }, { "id": 1159, "name": "libpthread-2.19.so@5900", "module": "libpthread-2.19.so" }, { "id": 1164, "name": "libpthread-2.19.so@58d0", "module": "libpthread-2.19.so" }, { "id": 1167, "name": "do_clone.constprop.3", "module": "libpthread-2.19.so" }, { "id": 1168, "name": "libpthread-2.19.so@5550", "module": "libpthread-2.19.so" }, { "id": 1171, "name": "stub_clone", "module": "<.text.kernel>" }, { "id": 1172, "name": "SyS_clone", "module": "<.text.kernel>" }, { "id": 1173, "name": "do_fork", "module": "<.text.kernel>" }, { "id": 1174, "name": "copy_process.part.27", "module": "<.text.kernel>" }, { "id": 1175, "name": "task_active_pid_ns", "module": "<.text.kernel>" }, { "id": 1176, "name": "security_task_create", "module": "<.text.kernel>" }, { "id": 1177, "name": "cap_task_create", "module": "<.text.kernel>" }, { "id": 1178, "name": "tsk_fork_get_node", "module": "<.text.kernel>" }, { "id": 1179, "name": "kmem_cache_alloc_node", "module": "<.text.kernel>" }, { "id": 1180, "name": "arch_dup_task_struct", "module": "<.text.kernel>" }, { "id": 1181, "name": "get_random_int", "module": "<.text.kernel>" }, { "id": 1182, "name": "account_kernel_stack", "module": "<.text.kernel>" }, { "id": 1183, "name": "mod_zone_page_state", "module": "<.text.kernel>" }, { "id": 1184, "name": "ftrace_graph_init_task", "module": "<.text.kernel>" }, { "id": 1185, "name": "get_seccomp_filter", "module": "<.text.kernel>" }, { "id": 1186, "name": "copy_creds", "module": "<.text.kernel>" }, { "id": 1187, "name": "__delayacct_tsk_init", "module": "<.text.kernel>" }, { "id": 1188, "name": "acct_clear_integrals", "module": "<.text.kernel>" }, { "id": 1189, "name": "ktime_get_ts", "module": "<.text.kernel>" }, { "id": 1190, "name": "monotonic_to_bootbased", "module": "<.text.kernel>" }, { "id": 1191, "name": "down_read", "module": "<.text.kernel>" }, { "id": 1192, "name": "cgroup_fork", "module": "<.text.kernel>" }, { "id": 1193, "name": "mpol_fix_fork_child_flag", "module": "<.text.kernel>" }, { "id": 1194, "name": "sched_fork", "module": "<.text.kernel>" }, { "id": 1195, "name": "__sched_fork", "module": "<.text.kernel>" }, { "id": 1196, "name": "task_fork_fair", "module": "<.text.kernel>" }, { "id": 1197, "name": "sched_slice.isra.45", "module": "<.text.kernel>" }, { "id": 1198, "name": "set_task_cpu", "module": "<.text.kernel>" }, { "id": 1199, "name": "perf_event_init_task", "module": "<.text.kernel>" }, { "id": 1200, "name": "__mutex_init", "module": "<.text.kernel>" }, { "id": 1201, "name": "perf_event_init_context", "module": "<.text.kernel>" }, { "id": 1202, "name": "audit_alloc", "module": "<.text.kernel>" }, { "id": 1203, "name": "copy_semundo", "module": "<.text.kernel>" }, { "id": 1204, "name": "copy_namespaces", "module": "<.text.kernel>" }, { "id": 1205, "name": "copy_thread", "module": "<.text.kernel>" }, { "id": 1206, "name": "alloc_pid", "module": "<.text.kernel>" }, { "id": 1207, "name": "user_disable_single_step", "module": "<.text.kernel>" }, { "id": 1208, "name": "clear_ti_thread_flag", "module": "<.text.kernel>" }, { "id": 1209, "name": "clear_all_latency_tracing", "module": "<.text.kernel>" }, { "id": 1210, "name": "_raw_write_lock_irq", "module": "<.text.kernel>" }, { "id": 1211, "name": "attach_pid", "module": "<.text.kernel>" }, { "id": 1212, "name": "__raw_write_unlock_irq.constprop.30", "module": "<.text.kernel>" }, { "id": 1213, "name": "proc_fork_connector", "module": "<.text.kernel>" }, { "id": 1214, "name": "cgroup_post_fork", "module": "<.text.kernel>" }, { "id": 1215, "name": "_raw_write_lock", "module": "<.text.kernel>" }, { "id": 1216, "name": "freezer_fork", "module": "<.text.kernel>" }, { "id": 1217, "name": "perf_event_fork", "module": "<.text.kernel>" }, { "id": 1218, "name": "perf_event_task", "module": "<.text.kernel>" }, { "id": 1219, "name": "uprobe_copy_process", "module": "<.text.kernel>" }, { "id": 1220, "name": "__put_user_4", "module": "<.text.kernel>" }, { "id": 1221, "name": "wake_up_new_task", "module": "<.text.kernel>" }, { "id": 1222, "name": "__bitmap_intersects", "module": "<.text.kernel>" }, { "id": 1223, "name": "cpumask_next_and", "module": "<.text.kernel>" }, { "id": 1224, "name": "migrate_task_rq_fair", "module": "<.text.kernel>" }, { "id": 1225, "name": "init_task_runnable_average", "module": "<.text.kernel>" }, { "id": 1226, "name": "(unsigned short, vector, unsigned long, signed char, ..., size, void)", "module": "main" }, { "id": 1227, "name": "(unsigned short, vector, unsigned long, signed char, ..., capacity, void)", "module": "main" }, { "id": 1228, "name": "std::vector(unsigned long, signed char, ..., push_back, const & ...)", "module": "main" }, { "id": 1229, "name": "std::vector(unsigned long, signed char, ..., _M_emplace_back_aux, const & ..., void)", "module": "main" }, { "id": 1230, "name": "(unsigned short, vector, unsigned long, signed char, ..., _M_check_len, unsigned long, const char)", "module": "main" }, { "id": 1231, "name": "(unsigned short, vector, unsigned long, signed char, ..., max_size, void)", "module": "main" }, { "id": 1232, "name": "(unsigned short, _Vector_base, unsigned long, signed char, ..., _M_get_Tp_allocator, void)", "module": "main" }, { "id": 1233, "name": "std::allocator_traits(signed char, unsigned long, max_size, const & ...)", "module": "main" }, { "id": 1234, "name": "_ZNSt16allocator_traitsISaIPmEE11_S_max_sizeIKS1_EENSt9enable_ifIXsrNS2_16__maxsize_helperIT_EE5valueEmE4typeERS7_", "module": "main" }, { "id": 1235, "name": "(__gnu_cxx, new_allocator, unsigned long, max_size, void)", "module": "main" }, { "id": 1236, "name": "std::max(unsigned long, ..., ...)", "module": "main" }, { "id": 1237, "name": "std::_Vector_base(unsigned long, signed char, ..., _M_allocate, unsigned long)", "module": "main" }, { "id": 1238, "name": "__gnu_cxx::new_allocator(unsigned long, allocate, unsigned long, const void)", "module": "main" }, { "id": 1239, "name": "std::forward(unsigned long, unsigned short, remove_reference, ..., type)", "module": "main" }, { "id": 1240, "name": "_ZNSt16allocator_traitsISaIPmEE9constructIS0_IRKS0_EEEDTcl12_S_constructfp_fp0_spcl7forwardIT0_Efp1_EEERS1_PT_DpOS6_", "module": "main" }, { "id": 1241, "name": "std::allocator_traits(signed char, unsigned long, _S_construct, ..., const & ..., unsigned short, enable_if, short, ..., __construct_helper, , value, void, type, & ..., _DpOS9_)", "module": "main" }, { "id": 1242, "name": "__gnu_cxx::new_allocator(unsigned long, construct, ..., const & ..., void, )", "module": "main" }, { "id": 1243, "name": "new(unsigned long, void)", "module": "main" }, { "id": 1244, "name": "std::_Vector_base(unsigned long, signed char, ..., _M_get_Tp_allocator, void)", "module": "main" }, { "id": 1245, "name": "std::__uninitialized_move_if_noexcept_a(unsigned long, ..., signed char, ..., , ..., ..., _)", "module": "main" }, { "id": 1246, "name": "std::__make_move_if_noexcept_iterator(unsigned long, unsigned short, move_iterator, ..., )", "module": "main" }, { "id": 1247, "name": "std::move_iterator(unsigned long, E, ...)", "module": "main" }, { "id": 1248, "name": "std::__uninitialized_copy_a(unsigned short, move_iterator, unsigned long, ..., ..., , ..., ..., signed char, _)", "module": "main" }, { "id": 1249, "name": "std::uninitialized_copy(unsigned short, move_iterator, unsigned long, ..., , ..., ...)", "module": "main" }, { "id": 1250, "name": "_ZNSt20__uninitialized_copyILb1EE13__uninit_copyISt13move_iteratorIPPmES4_EET0_T_S7_S6_", "module": "main" }, { "id": 1251, "name": "std::copy(unsigned short, move_iterator, unsigned long, ..., , ..., ...)", "module": "main" }, { "id": 1252, "name": "std::__miter_base(unsigned short, move_iterator, unsigned long, unsigned short, _Miter_base, iterator_type, ...)", "module": "main" }, { "id": 1253, "name": "std::_Iter_base(unsigned short, move_iterator, unsigned long, bool, E, _S_base, ...)", "module": "main" }, { "id": 1254, "name": "(unsigned short, move_iterator, unsigned long, base, void)", "module": "main" }, { "id": 1255, "name": "std::__copy_move_a2(bool, E, unsigned long, ..., _, , ..., ...)", "module": "main" }, { "id": 1256, "name": "std::__niter_base(unsigned long, unsigned short, _Niter_base, iterator_type, ...)", "module": "main" }, { "id": 1257, "name": "std::_Iter_base(unsigned long, bool, , _S_base, ...)", "module": "main" }, { "id": 1258, "name": "std::__copy_move_a(bool, E, unsigned long, ..., _, , ..., ...)", "module": "main" }, { "id": 1259, "name": "_ZNSt11__copy_moveILb1ELb1ESt26random_access_iterator_tagE8__copy_mIPmEEPT_PKS4_S7_S5_", "module": "main" }, { "id": 1260, "name": "std::_Destroy(unsigned long, ..., void, ..., signed char, )", "module": "main" }, { "id": 1261, "name": "std::_Destroy(unsigned long, void, ...)", "module": "main" }, { "id": 1262, "name": "std::_Destroy_aux(bool, E, __destroy, unsigned long, void, ...)", "module": "main" }, { "id": 1263, "name": "std::_Vector_base(unsigned long, signed char, ..., _M_deallocate, ..., unsigned long)", "module": "main" }, { "id": 1264, "name": "__anon_vma_interval_tree_augment_rotate", "module": "<.text.kernel>" }, { "id": 1266, "name": "libc-2.19.so@108120", "module": "libc-2.19.so" }, { "id": 1268, "name": "libc-2.19.so@108150", "module": "libc-2.19.so" }, { "id": 1269, "name": "futex_wake", "module": "<.text.kernel>" }, { "id": 1270, "name": "main@400a70", "module": "main" }, { "id": 1277, "name": "__gnu_cxx::new_allocator(unsigned long, deallocate, ..., unsigned long)", "module": "main" }, { "id": 1278, "name": "main@4009f0", "module": "main" }, { "id": 1287, "name": "call_rwsem_wake", "module": "<.text.kernel>" }, { "id": 1288, "name": "rwsem_wake", "module": "<.text.kernel>" }, { "id": 1289, "name": "__rwsem_do_wake", "module": "<.text.kernel>" }, { "id": 1290, "name": "call_rwsem_down_write_failed", "module": "<.text.kernel>" }, { "id": 1291, "name": "rwsem_down_write_failed", "module": "<.text.kernel>" }, { "id": 1292, "name": "idle_balance", "module": "<.text.kernel>" }, { "id": 1293, "name": "load_balance", "module": "<.text.kernel>" }, { "id": 1294, "name": "find_busiest_group", "module": "<.text.kernel>" }, { "id": 1295, "name": "update_group_power", "module": "<.text.kernel>" }, { "id": 1296, "name": "sched_cfs_period_timer", "module": "<.text.kernel>" }, { "id": 1297, "name": "pick_next_task_stop", "module": "<.text.kernel>" }, { "id": 1298, "name": "pick_next_task_rt", "module": "<.text.kernel>" }, { "id": 1299, "name": "pick_next_task_idle", "module": "<.text.kernel>" }, { "id": 1300, "name": "post_schedule_idle", "module": "<.text.kernel>" }, { "id": 1301, "name": "idle_enter_fair", "module": "<.text.kernel>" }, { "id": 1302, "name": "schedule_preempt_disabled", "module": "<.text.kernel>" }, { "id": 1304, "name": "tick_nohz_idle_enter", "module": "<.text.kernel>" }, { "id": 1305, "name": "set_cpu_sd_state_idle", "module": "<.text.kernel>" }, { "id": 1306, "name": "__tick_nohz_idle_enter", "module": "<.text.kernel>" }, { "id": 1307, "name": "sched_clock_idle_sleep_event", "module": "<.text.kernel>" }, { "id": 1308, "name": "tick_nohz_stop_sched_tick", "module": "<.text.kernel>" }, { "id": 1309, "name": "timekeeping_max_deferment", "module": "<.text.kernel>" }, { "id": 1310, "name": "rcu_needs_cpu", "module": "<.text.kernel>" }, { "id": 1311, "name": "rcu_cpu_has_callbacks", "module": "<.text.kernel>" }, { "id": 1312, "name": "irq_work_needs_cpu", "module": "<.text.kernel>" }, { "id": 1313, "name": "get_next_timer_interrupt", "module": "<.text.kernel>" }, { "id": 1314, "name": "hrtimer_get_next_event", "module": "<.text.kernel>" }, { "id": 1315, "name": "arch_cpu_idle_enter", "module": "<.text.kernel>" }, { "id": 1316, "name": "local_touch_nmi", "module": "<.text.kernel>" }, { "id": 1317, "name": "atomic_notifier_call_chain", "module": "<.text.kernel>" }, { "id": 1318, "name": "tick_check_broadcast_expired", "module": "<.text.kernel>" }, { "id": 1319, "name": "rcu_idle_enter", "module": "<.text.kernel>" }, { "id": 1320, "name": "arch_cpu_idle", "module": "<.text.kernel>" }, { "id": 1321, "name": "cpuidle_idle_call", "module": "<.text.kernel>" }, { "id": 1322, "name": "cpuidle_get_cpu_driver", "module": "<.text.kernel>" }, { "id": 1323, "name": "menu_select", "module": "<.text.kernel>" }, { "id": 1324, "name": "pm_qos_request", "module": "<.text.kernel>" }, { "id": 1325, "name": "tick_nohz_get_sleep_length", "module": "<.text.kernel>" }, { "id": 1326, "name": "ns_to_timespec", "module": "<.text.kernel>" }, { "id": 1327, "name": "nr_iowait_cpu", "module": "<.text.kernel>" }, { "id": 1328, "name": "this_cpu_load", "module": "<.text.kernel>" }, { "id": 1329, "name": "int_sqrt", "module": "<.text.kernel>" }, { "id": 1330, "name": "menu_reflect", "module": "<.text.kernel>" }, { "id": 1331, "name": "reschedule_interrupt", "module": "<.text.kernel>" }, { "id": 1332, "name": "smp_reschedule_interrupt", "module": "<.text.kernel>" }, { "id": 1333, "name": "scheduler_ipi", "module": "<.text.kernel>" }, { "id": 1334, "name": "rcu_idle_exit", "module": "<.text.kernel>" }, { "id": 1335, "name": "arch_cpu_idle_exit", "module": "<.text.kernel>" }, { "id": 1336, "name": "tick_nohz_idle_exit", "module": "<.text.kernel>" }, { "id": 1337, "name": "tick_nohz_stop_idle", "module": "<.text.kernel>" }, { "id": 1338, "name": "update_ts_time_stats", "module": "<.text.kernel>" }, { "id": 1339, "name": "sched_clock_idle_wakeup_event", "module": "<.text.kernel>" }, { "id": 1340, "name": "touch_softlockup_watchdog", "module": "<.text.kernel>" }, { "id": 1341, "name": "pre_schedule_idle", "module": "<.text.kernel>" }, { "id": 1342, "name": "idle_exit_fair", "module": "<.text.kernel>" }, { "id": 1343, "name": "put_prev_task_idle", "module": "<.text.kernel>" }, { "id": 1344, "name": "vtime_account_idle", "module": "<.text.kernel>" }, { "id": 1345, "name": "native_smp_send_reschedule", "module": "<.text.kernel>" }, { "id": 1346, "name": "flat_send_IPI_mask", "module": "<.text.kernel>" }, { "id": 1348, "name": "get_partial_node.isra.54", "module": "<.text.kernel>" }, { "id": 1349, "name": "std::vector(unsigned long, signed char, ..., back, void)", "module": "main" }, { "id": 1350, "name": "std::vector(unsigned long, signed char, ..., end, void)", "module": "main" }, { "id": 1351, "name": "__gnu_cxx::__normal_iterator(unsigned long, unsigned short, vector, ..., signed char, ..., E, const & ...)", "module": "main" }, { "id": 1352, "name": "(__gnu_cxx, __normal_iterator, unsigned long, unsigned short, vector, ..., signed char, ..., unsigned long, int, const & long)", "module": "main" }, { "id": 1353, "name": "(__gnu_cxx, __normal_iterator, unsigned long, unsigned short, vector, ..., signed char, ..., double, float80, void)", "module": "main" }, { "id": 1354, "name": "main@400a60", "module": "main" }, { "id": 1357, "name": "_pthread_cleanup_push", "module": "libpthread-2.19.so" }, { "id": 1358, "name": "__pthread_enable_asynccancel", "module": "libpthread-2.19.so" }, { "id": 1359, "name": "get_user_pages_fast", "module": "<.text.kernel>" }, { "id": 1360, "name": "gup_pud_range", "module": "<.text.kernel>" }, { "id": 1361, "name": "gup_pte_range", "module": "<.text.kernel>" }, { "id": 1362, "name": "futex_wait_queue_me", "module": "<.text.kernel>" }, { "id": 1363, "name": "plist_add", "module": "<.text.kernel>" }, { "id": 1364, "name": "cpuidle_enter_state", "module": "<.text.kernel>" }, { "id": 1365, "name": "acpi_idle_enter_simple", "module": "<.text.kernel>" }, { "id": 1366, "name": "current_set_polling_and_test", "module": "<.text.kernel>" }, { "id": 1367, "name": "lapic_timer_state_broadcast", "module": "<.text.kernel>" }, { "id": 1368, "name": "acpi_idle_do_entry", "module": "<.text.kernel>" }, { "id": 1369, "name": "acpi_processor_ffh_cstate_enter", "module": "<.text.kernel>" }, { "id": 1370, "name": "mwait_idle_with_hints", "module": "<.text.kernel>" }, { "id": 1371, "name": "ns_to_timeval", "module": "<.text.kernel>" }, { "id": 1460, "name": "libc-2.19.so@1f420", "module": "libc-2.19.so" }, { "id": 1532, "name": "do_exit", "module": "<.text.kernel>" }, { "id": 1533, "name": "profile_task_exit", "module": "<.text.kernel>" }, { "id": 1534, "name": "exit_signals", "module": "<.text.kernel>" }, { "id": 1535, "name": "hrtimer_cancel", "module": "<.text.kernel>" }, { "id": 1536, "name": "hrtimer_try_to_cancel", "module": "<.text.kernel>" }, { "id": 1537, "name": "lock_hrtimer_base.isra.19", "module": "<.text.kernel>" }, { "id": 1539, "name": "acct_collect", "module": "<.text.kernel>" }, { "id": 1541, "name": "taskstats_exit", "module": "<.text.kernel>" }, { "id": 1542, "name": "__delayacct_add_tsk", "module": "<.text.kernel>" }, { "id": 1543, "name": "task_cputime_scaled", "module": "<.text.kernel>" }, { "id": 1544, "name": "mm_release", "module": "<.text.kernel>" }, { "id": 1545, "name": "exit_robust_list", "module": "<.text.kernel>" }, { "id": 1546, "name": "__get_user_8", "module": "<.text.kernel>" }, { "id": 1547, "name": "uprobe_free_utask", "module": "<.text.kernel>" }, { "id": 1548, "name": "native_load_gs_index", "module": "<.text.kernel>" }, { "id": 1550, "name": "mm_update_next_owner", "module": "<.text.kernel>" }, { "id": 1551, "name": "mmput", "module": "<.text.kernel>" }, { "id": 1561, "name": "leave_mm", "module": "<.text.kernel>" }, { "id": 1562, "name": "rwsem_down_read_failed", "module": "<.text.kernel>" }, { "id": 1563, "name": "call_rwsem_down_read_failed", "module": "<.text.kernel>" }, { "id": 1565, "name": "libc-2.19.so@165f90", "module": "libc-2.19.so" }, { "id": 1566, "name": "nohz_balance_enter_idle", "module": "<.text.kernel>" }, { "id": 1567, "name": "calc_load_enter_idle", "module": "<.text.kernel>" }, { "id": 1568, "name": "hrtimer_start", "module": "<.text.kernel>" }, { "id": 1569, "name": "__hrtimer_start_range_ns", "module": "<.text.kernel>" }, { "id": 1570, "name": "hrtimer_force_reprogram", "module": "<.text.kernel>" }, { "id": 1571, "name": "rcu_sysidle_force_exit", "module": "<.text.kernel>" }, { "id": 1572, "name": "update_cpu_load_nohz", "module": "<.text.kernel>" }, { "id": 1573, "name": "calc_load_exit_idle", "module": "<.text.kernel>" }, { "id": 1574, "name": "tick_nohz_restart", "module": "<.text.kernel>" }, { "id": 1575, "name": "hrtimer_start_range_ns", "module": "<.text.kernel>" }, { "id": 1576, "name": "ptep_set_access_flags", "module": "<.text.kernel>" }, { "id": 1578, "name": "libc-2.19.so@166080", "module": "libc-2.19.so" }, { "id": 1580, "name": "libc-2.19.so@1660a0", "module": "libc-2.19.so" }, { "id": 1581, "name": "libc-2.19.so@166170", "module": "libc-2.19.so" }, { "id": 1582, "name": "start_thread", "module": "libpthread-2.19.so" }, { "id": 1583, "name": "libpthread-2.19.so@58b0", "module": "libpthread-2.19.so" }, { "id": 1586, "name": "SyS_madvise", "module": "<.text.kernel>" }, { "id": 1587, "name": "find_vma_prev", "module": "<.text.kernel>" }, { "id": 1588, "name": "blk_start_plug", "module": "<.text.kernel>" }, { "id": 1589, "name": "zap_page_range", "module": "<.text.kernel>" }, { "id": 1590, "name": "blk_finish_plug", "module": "<.text.kernel>" }, { "id": 1591, "name": "blk_flush_plug_list", "module": "<.text.kernel>" }, { "id": 1592, "name": "SyS_exit", "module": "<.text.kernel>" }, { "id": 1593, "name": "probe_sched_process_exit", "module": ".text.simple_pt" }, { "id": 1594, "name": "ftrace_raw_event_exit", "module": ".text.simple_pt" }, { "id": 1595, "name": "exit_sem", "module": "<.text.kernel>" }, { "id": 1596, "name": "exit_shm", "module": "<.text.kernel>" }, { "id": 1597, "name": "idr_for_each", "module": "<.text.kernel>" }, { "id": 1598, "name": "shm_try_destroy_current", "module": "<.text.kernel>" }, { "id": 1599, "name": "exit_files", "module": "<.text.kernel>" }, { "id": 1600, "name": "put_files_struct", "module": "<.text.kernel>" }, { "id": 1601, "name": "exit_fs", "module": "<.text.kernel>" }, { "id": 1602, "name": "exit_task_namespaces", "module": "<.text.kernel>" }, { "id": 1603, "name": "switch_task_namespaces", "module": "<.text.kernel>" }, { "id": 1604, "name": "exit_thread", "module": "<.text.kernel>" }, { "id": 1605, "name": "perf_event_exit_task", "module": "<.text.kernel>" }, { "id": 1606, "name": "cgroup_exit", "module": "<.text.kernel>" }, { "id": 1607, "name": "cpu_cgroup_exit", "module": "<.text.kernel>" }, { "id": 1608, "name": "sched_move_task", "module": "<.text.kernel>" }, { "id": 1609, "name": "task_rq_lock", "module": "<.text.kernel>" }, { "id": 1610, "name": "task_wants_autogroup", "module": "<.text.kernel>" }, { "id": 1611, "name": "task_move_group_fair", "module": "<.text.kernel>" }, { "id": 1612, "name": "set_curr_task_fair", "module": "<.text.kernel>" }, { "id": 1613, "name": "perf_cgroup_exit", "module": "<.text.kernel>" }, { "id": 1614, "name": "task_function_call", "module": "<.text.kernel>" }, { "id": 1615, "name": "task_curr", "module": "<.text.kernel>" }, { "id": 1616, "name": "smp_call_function_single", "module": "<.text.kernel>" }, { "id": 1617, "name": "remote_function", "module": "<.text.kernel>" }, { "id": 1618, "name": "__perf_cgroup_move", "module": "<.text.kernel>" }, { "id": 1619, "name": "perf_cgroup_switch", "module": "<.text.kernel>" }, { "id": 1620, "name": "__put_css_set", "module": "<.text.kernel>" }, { "id": 1621, "name": "proc_exit_connector", "module": "<.text.kernel>" }, { "id": 1622, "name": "flush_ptrace_hw_breakpoint", "module": "<.text.kernel>" }, { "id": 1623, "name": "unregister_hw_breakpoint", "module": "<.text.kernel>" }, { "id": 1624, "name": "exit_ptrace", "module": "<.text.kernel>" }, { "id": 1625, "name": "__raw_write_unlock_irq.constprop.11", "module": "<.text.kernel>" }, { "id": 1626, "name": "release_task", "module": "<.text.kernel>" }, { "id": 1627, "name": "proc_flush_task", "module": "<.text.kernel>" }, { "id": 1628, "name": "snprintf", "module": "<.text.kernel>" }, { "id": 1629, "name": "vsnprintf", "module": "<.text.kernel>" }, { "id": 1630, "name": "format_decode", "module": "<.text.kernel>" }, { "id": 1631, "name": "number.isra.1", "module": "<.text.kernel>" }, { "id": 1632, "name": "put_dec", "module": "<.text.kernel>" }, { "id": 1633, "name": "put_dec_trunc8", "module": "<.text.kernel>" }, { "id": 1634, "name": "d_hash_and_lookup", "module": "<.text.kernel>" }, { "id": 1635, "name": "full_name_hash", "module": "<.text.kernel>" }, { "id": 1636, "name": "d_lookup", "module": "<.text.kernel>" }, { "id": 1637, "name": "posix_cpu_timers_exit", "module": "<.text.kernel>" }, { "id": 1638, "name": "add_device_randomness", "module": "<.text.kernel>" }, { "id": 1639, "name": "_mix_pool_bytes", "module": "<.text.kernel>" }, { "id": 1640, "name": "cleanup_timers.isra.2", "module": "<.text.kernel>" }, { "id": 1641, "name": "task_gtime", "module": "<.text.kernel>" }, { "id": 1642, "name": "detach_pid", "module": "<.text.kernel>" }, { "id": 1643, "name": "__change_pid", "module": "<.text.kernel>" }, { "id": 1644, "name": "free_pid", "module": "<.text.kernel>" }, { "id": 1645, "name": "free_pidmap.isra.0", "module": "<.text.kernel>" }, { "id": 1646, "name": "flush_sigqueue", "module": "<.text.kernel>" }, { "id": 1647, "name": "__cleanup_sighand", "module": "<.text.kernel>" }, { "id": 1648, "name": "release_thread", "module": "<.text.kernel>" }, { "id": 1649, "name": "exit_rcu", "module": "<.text.kernel>" }, { "id": 1650, "name": "kprobe_flush_task", "module": "<.text.kernel>" }, { "id": 1651, "name": "can_migrate_task", "module": "<.text.kernel>" }, { "id": 1652, "name": "update_cfs_rq_h_load", "module": "<.text.kernel>" }, { "id": 1653, "name": "move_task", "module": "<.text.kernel>" }, { "id": 1655, "name": "schedule_tail", "module": "<.text.kernel>" }, { "id": 1660, "name": "libpthread-2.19.so@5820", "module": "libpthread-2.19.so" }, { "id": 1661, "name": "libpthread-2.19.so@5700", "module": "libpthread-2.19.so" }, { "id": 1662, "name": "libpthread-2.19.so@56d0", "module": "libpthread-2.19.so" }, { "id": 1663, "name": "__nptl_deallocate_tsd", "module": "libpthread-2.19.so" }, { "id": 1664, "name": "libpthread-2.19.so@5760", "module": "libpthread-2.19.so" }, { "id": 1665, "name": "acpi_idle_enter_bm", "module": "<.text.kernel>" }, { "id": 1667, "name": "native_set_ldt", "module": "<.text.kernel>" }, { "id": 1668, "name": "thread_work_1()", "module": "main" }, { "id": 1669, "name": "main@4009d0", "module": "main" }, { "id": 1671, "name": "strlen", "module": "libc-2.19.so" }, { "id": 1672, "name": "wake_futex", "module": "<.text.kernel>" }, { "id": 1673, "name": "__unqueue_futex", "module": "<.text.kernel>" }, { "id": 1674, "name": "plist_del", "module": "<.text.kernel>" }, { "id": 1675, "name": "wake_up_state", "module": "<.text.kernel>" }, { "id": 1678, "name": "rcu_start_future_gp", "module": "<.text.kernel>" }, { "id": 1679, "name": "thread_work_0()", "module": "main" }, { "id": 1681, "name": "__mmdrop", "module": "<.text.kernel>" }, { "id": 1682, "name": "pgd_free", "module": "<.text.kernel>" }, { "id": 1683, "name": "destroy_context", "module": "<.text.kernel>" }, { "id": -1, "name": "<Unknown>", "module": "<Unknown>" }],
        links: [{ "source": 670, "target": 642, "value": 114 }, { "source": 669, "target": 670, "value": 188 }, { "source": 285, "target": 669, "value": 3004 }, { "source": 1653, "target": 285, "value": 962 }, { "source": 1293, "target": 1653, "value": 11156 }, { "source": 1292, "target": 1293, "value": 97932 }, { "source": 683, "target": 1292, "value": 130022 }, { "source": 704, "target": 683, "value": 442158 }, { "source": 1532, "target": 704, "value": 216332 }, { "source": 1592, "target": 1532, "value": 769492 }, { "source": 160, "target": 1592, "value": 769636 }, { "source": 1043, "target": 1268, "value": 10144 }, { "source": 1040, "target": 1043, "value": 136456 }, { "source": 1084, "target": 1040, "value": 93538 }, { "source": 1582, "target": 1084, "value": 489812 }, { "source": 1043, "target": 1266, "value": 2750 }, { "source": 50, "target": 51, "value": 10296 }, { "source": 49, "target": 50, "value": 14080 }, { "source": 40, "target": 49, "value": 15620 }, { "source": 682, "target": 40, "value": 1540 }, { "source": 1331, "target": 682, "value": 11582 }, { "source": 41, "target": 48, "value": 1540 }, { "source": 40, "target": 41, "value": 60500 }, { "source": 22, "target": 23, "value": 9720 }, { "source": 47, "target": 22, "value": 27800 }, { "source": 45, "target": 47, "value": 16982 }, { "source": 43, "target": 45, "value": 32052 }, { "source": 41, "target": 43, "value": 52140 }, { "source": 45, "target": 46, "value": 3014 }, { "source": 20, "target": 21, "value": 11646 }, { "source": 19, "target": 20, "value": 40114 }, { "source": 18, "target": 19, "value": 43888 }, { "source": 17, "target": 18, "value": 38226 }, { "source": 44, "target": 17, "value": 40868 }, { "source": 43, "target": 44, "value": 27400 }, { "source": 41, "target": 42, "value": 2420 }, { "source": 703, "target": 48, "value": 1260 }, { "source": 702, "target": 703, "value": 11070 }, { "source": 701, "target": 702, "value": 29954 }, { "source": 683, "target": 701, "value": 28022 }, { "source": 683, "target": 704, "value": 10024 }, { "source": 682, "target": 683, "value": 20160 }, { "source": 703, "target": 18, "value": 5130 }, { "source": 703, "target": 42, "value": 1980 }, { "source": 603, "target": 48, "value": 378 }, { "source": 702, "target": 603, "value": 13420 }, { "source": 603, "target": 43, "value": 11422 }, { "source": 603, "target": 42, "value": 594 }, { "source": 698, "target": 0, "value": 416 }, { "source": 683, "target": 698, "value": 11436 }, { "source": 698, "target": 700, "value": 1196 }, { "source": 698, "target": 699, "value": 552 }, { "source": 683, "target": 697, "value": 666 }, { "source": 683, "target": 696, "value": 1036 }, { "source": 292, "target": 293, "value": 1584 }, { "source": 291, "target": 292, "value": 3168 }, { "source": 695, "target": 291, "value": 7202 }, { "source": 683, "target": 695, "value": 8830 }, { "source": 540, "target": 543, "value": 324 }, { "source": 727, "target": 540, "value": 4114 }, { "source": 690, "target": 727, "value": 16966 }, { "source": 683, "target": 690, "value": 50618 }, { "source": 541, "target": 542, "value": 2700 }, { "source": 540, "target": 541, "value": 10476 }, { "source": 727, "target": 539, "value": 136 }, { "source": 535, "target": 536, "value": 3888 }, { "source": 727, "target": 535, "value": 782 }, { "source": 531, "target": 537, "value": 4104 }, { "source": 530, "target": 531, "value": 29484 }, { "source": 727, "target": 530, "value": 9758 }, { "source": 531, "target": 535, "value": 2484 }, { "source": 534, "target": 19, "value": 3996 }, { "source": 533, "target": 534, "value": 4536 }, { "source": 532, "target": 533, "value": 15552 }, { "source": 531, "target": 532, "value": 19548 }, { "source": 690, "target": 693, "value": 1580 }, { "source": 690, "target": 691, "value": 2268 }, { "source": 688, "target": 613, "value": 1500 }, { "source": 686, "target": 688, "value": 7696 }, { "source": 683, "target": 686, "value": 12956 }, { "source": 688, "target": 612, "value": 1004 }, { "source": 688, "target": 689, "value": 2384 }, { "source": 686, "target": 687, "value": 924 }, { "source": 683, "target": 685, "value": 4308 }, { "source": 725, "target": 283, "value": 312 }, { "source": 724, "target": 725, "value": 50920 }, { "source": 723, "target": 724, "value": 39200 }, { "source": 683, "target": 723, "value": 34820 }, { "source": 726, "target": 279, "value": 11886 }, { "source": 725, "target": 726, "value": 42578 }, { "source": 726, "target": 640, "value": 2308 }, { "source": 726, "target": 643, "value": 3496 }, { "source": 726, "target": 687, "value": 1440 }, { "source": 726, "target": 277, "value": 2856 }, { "source": 276, "target": 641, "value": 2272 }, { "source": 726, "target": 276, "value": 8218 }, { "source": 276, "target": 640, "value": 3856 }, { "source": 273, "target": 18, "value": 12540 }, { "source": 724, "target": 273, "value": 5538 }, { "source": 683, "target": 665, "value": 962 }, { "source": 683, "target": 684, "value": 1406 }, { "source": 711, "target": 295, "value": 136 }, { "source": 683, "target": 711, "value": 680 }, { "source": 711, "target": 259, "value": 144 }, { "source": 683, "target": 295, "value": 680 }, { "source": 683, "target": 1678, "value": 104 }, { "source": 683, "target": 259, "value": 720 }, { "source": 712, "target": 295, "value": 68 }, { "source": 683, "target": 712, "value": 256 }, { "source": 712, "target": 259, "value": 72 }, { "source": 887, "target": 540, "value": 6050 }, { "source": 690, "target": 887, "value": 24950 }, { "source": 887, "target": 539, "value": 200 }, { "source": 887, "target": 535, "value": 1150 }, { "source": 887, "target": 530, "value": 14350 }, { "source": 281, "target": 282, "value": 888 }, { "source": 685, "target": 281, "value": 488 }, { "source": 685, "target": 276, "value": 630 }, { "source": 65, "target": 48, "value": 1540 }, { "source": 39, "target": 65, "value": 55690 }, { "source": 682, "target": 39, "value": 746 }, { "source": 66, "target": 47, "value": 14770 }, { "source": 65, "target": 66, "value": 24450 }, { "source": 66, "target": 46, "value": 2420 }, { "source": 65, "target": 44, "value": 22000 }, { "source": 65, "target": 42, "value": 2420 }, { "source": 63, "target": 64, "value": 6260 }, { "source": 62, "target": 63, "value": 11220 }, { "source": 39, "target": 62, "value": 12760 }, { "source": 1332, "target": 1333, "value": 600 }, { "source": 1331, "target": 1332, "value": 940 }, { "source": 1332, "target": 598, "value": 120 }, { "source": 1321, "target": 1331, "value": 1288 }, { "source": 1320, "target": 1321, "value": 72626 }, { "source": 683, "target": 1320, "value": 72870 }, { "source": 1291, "target": 704, "value": 105408 }, { "source": 1290, "target": 1291, "value": 106752 }, { "source": 145, "target": 1290, "value": 106940 }, { "source": 1596, "target": 145, "value": 65592 }, { "source": 1532, "target": 1596, "value": 274212 }, { "source": 38, "target": 40, "value": 57750 }, { "source": 683, "target": 38, "value": 3552 }, { "source": 38, "target": 39, "value": 4542 }, { "source": 1655, "target": 701, "value": 9688 }, { "source": 683, "target": 1655, "value": 4130 }, { "source": 1292, "target": 42, "value": 462 }, { "source": 1293, "target": 48, "value": 140 }, { "source": 669, "target": 276, "value": 540 }, { "source": 274, "target": 283, "value": 312 }, { "source": 272, "target": 274, "value": 45402 }, { "source": 271, "target": 272, "value": 43378 }, { "source": 1653, "target": 271, "value": 4782 }, { "source": 275, "target": 280, "value": 1680 }, { "source": 274, "target": 275, "value": 36938 }, { "source": 275, "target": 279, "value": 5000 }, { "source": 275, "target": 278, "value": 3496 }, { "source": 275, "target": 277, "value": 3454 }, { "source": 276, "target": 642, "value": 3478 }, { "source": 275, "target": 276, "value": 5912 }, { "source": 275, "target": 281, "value": 3072 }, { "source": 272, "target": 273, "value": 5538 }, { "source": 1198, "target": 1224, "value": 496 }, { "source": 1653, "target": 1198, "value": 306 }, { "source": 1653, "target": 723, "value": 4950 }, { "source": 725, "target": 279, "value": 180 }, { "source": 279, "target": 278, "value": 1054 }, { "source": 279, "target": 643, "value": 1054 }, { "source": 279, "target": 276, "value": 7302 }, { "source": 726, "target": 613, "value": 126 }, { "source": 726, "target": 612, "value": 84 }, { "source": 726, "target": 689, "value": 210 }, { "source": 1293, "target": 1652, "value": 194 }, { "source": 1293, "target": 1651, "value": 1036 }, { "source": 1293, "target": 42, "value": 2552 }, { "source": 791, "target": 197, "value": 4258 }, { "source": 788, "target": 791, "value": 5702 }, { "source": 787, "target": 788, "value": 20304 }, { "source": 832, "target": 787, "value": 24696 }, { "source": 1583, "target": 832, "value": 4408 }, { "source": 1582, "target": 1583, "value": 161036 }, { "source": 366, "target": 197, "value": 2576 }, { "source": 788, "target": 366, "value": 5060 }, { "source": 1578, "target": 832, "value": 1366 }, { "source": 1664, "target": 1578, "value": 1988 }, { "source": 1582, "target": 1664, "value": 49294 }, { "source": 1223, "target": 123, "value": 21718 }, { "source": 1293, "target": 1223, "value": 6136 }, { "source": 1562, "target": 704, "value": 118338 }, { "source": 1563, "target": 1562, "value": 127366 }, { "source": 1191, "target": 1563, "value": 127576 }, { "source": 67, "target": 1191, "value": 62154 }, { "source": 61, "target": 67, "value": 183320 }, { "source": 56, "target": 61, "value": 231576 }, { "source": 1046, "target": 1068, "value": 12344 }, { "source": 1669, "target": 1046, "value": 320 }, { "source": 1679, "target": 1669, "value": 5576 }, { "source": 1582, "target": 1679, "value": 331798 }, { "source": 1669, "target": 1671, "value": 3312 }, { "source": 1669, "target": 832, "value": 1992 }, { "source": 1043, "target": 1069, "value": 976 }, { "source": 1679, "target": 1040, "value": 4070 }, { "source": 1043, "target": 1046, "value": 42182 }, { "source": 1043, "target": 1044, "value": 9704 }, { "source": 1043, "target": 1065, "value": 5564 }, { "source": 1043, "target": 1045, "value": 1708 }, { "source": 1679, "target": 1075, "value": 12 }, { "source": 1070, "target": 1071, "value": 4060 }, { "source": 1679, "target": 1070, "value": 2050 }, { "source": 1067, "target": 1046, "value": 284 }, { "source": 1043, "target": 1067, "value": 412 }, { "source": 1043, "target": 1063, "value": 124 }, { "source": 1679, "target": 1039, "value": 24 }, { "source": 658, "target": 0, "value": 448 }, { "source": 657, "target": 658, "value": 2128 }, { "source": 656, "target": 657, "value": 7392 }, { "source": 1569, "target": 656, "value": 3168 }, { "source": 1568, "target": 1569, "value": 18100 }, { "source": 1308, "target": 1568, "value": 18370 }, { "source": 1306, "target": 1308, "value": 30596 }, { "source": 1304, "target": 1306, "value": 36836 }, { "source": 683, "target": 1304, "value": 37952 }, { "source": 658, "target": 21, "value": 504 }, { "source": 608, "target": 21, "value": 1386 }, { "source": 615, "target": 608, "value": 2376 }, { "source": 657, "target": 615, "value": 3080 }, { "source": 263, "target": 295, "value": 544 }, { "source": 262, "target": 263, "value": 7184 }, { "source": 261, "target": 262, "value": 7212 }, { "source": 260, "target": 261, "value": 7284 }, { "source": 258, "target": 260, "value": 7630 }, { "source": 257, "target": 258, "value": 8280 }, { "source": 256, "target": 257, "value": 9532 }, { "source": 255, "target": 256, "value": 11688 }, { "source": 1644, "target": 255, "value": 10192 }, { "source": 1643, "target": 1644, "value": 12502 }, { "source": 1642, "target": 1643, "value": 13030 }, { "source": 1626, "target": 1642, "value": 13184 }, { "source": 1532, "target": 1626, "value": 82998 }, { "source": 263, "target": 294, "value": 922 }, { "source": 263, "target": 48, "value": 266 }, { "source": 290, "target": 291, "value": 1820 }, { "source": 284, "target": 290, "value": 1570 }, { "source": 270, "target": 284, "value": 8196 }, { "source": 263, "target": 270, "value": 31148 }, { "source": 287, "target": 288, "value": 1232 }, { "source": 284, "target": 287, "value": 1484 }, { "source": 1345, "target": 1346, "value": 740 }, { "source": 286, "target": 1345, "value": 1180 }, { "source": 669, "target": 286, "value": 320 }, { "source": 284, "target": 285, "value": 3570 }, { "source": 669, "target": 671, "value": 68 }, { "source": 270, "target": 271, "value": 22258 }, { "source": 263, "target": 42, "value": 8780 }, { "source": 265, "target": 268, "value": 906 }, { "source": 263, "target": 265, "value": 11022 }, { "source": 265, "target": 123, "value": 9340 }, { "source": 265, "target": 1222, "value": 1680 }, { "source": 265, "target": 718, "value": 1754 }, { "source": 265, "target": 267, "value": 1034 }, { "source": 265, "target": 266, "value": 1034 }, { "source": 263, "target": 264, "value": 532 }, { "source": 263, "target": 259, "value": 576 }, { "source": 1291, "target": 665, "value": 260 }, { "source": 1302, "target": 683, "value": 35176 }, { "source": 683, "target": 1302, "value": 35336 }, { "source": 1344, "target": 44, "value": 3674 }, { "source": 702, "target": 1344, "value": 4186 }, { "source": 683, "target": 1343, "value": 100 }, { "source": 1341, "target": 1342, "value": 760 }, { "source": 683, "target": 1341, "value": 980 }, { "source": 1569, "target": 295, "value": 748 }, { "source": 1575, "target": 1569, "value": 5824 }, { "source": 1574, "target": 1575, "value": 5922 }, { "source": 1336, "target": 1574, "value": 9996 }, { "source": 683, "target": 1336, "value": 14120 }, { "source": 655, "target": 282, "value": 858 }, { "source": 654, "target": 655, "value": 3396 }, { "source": 1569, "target": 654, "value": 4540 }, { "source": 1537, "target": 259, "value": 1044 }, { "source": 1569, "target": 1537, "value": 1760 }, { "source": 1574, "target": 653, "value": 140 }, { "source": 1536, "target": 295, "value": 238 }, { "source": 1535, "target": 1536, "value": 3332 }, { "source": 1574, "target": 1535, "value": 3514 }, { "source": 1570, "target": 656, "value": 4896 }, { "source": 610, "target": 1570, "value": 6672 }, { "source": 1536, "target": 610, "value": 1946 }, { "source": 611, "target": 613, "value": 950 }, { "source": 610, "target": 611, "value": 2422 }, { "source": 611, "target": 612, "value": 544 }, { "source": 1536, "target": 1537, "value": 560 }, { "source": 1336, "target": 1340, "value": 84 }, { "source": 1336, "target": 1573, "value": 140 }, { "source": 1336, "target": 1572, "value": 196 }, { "source": 1336, "target": 617, "value": 168 }, { "source": 1339, "target": 1340, "value": 132 }, { "source": 1337, "target": 1339, "value": 380 }, { "source": 1336, "target": 1337, "value": 1480 }, { "source": 1339, "target": 638, "value": 88 }, { "source": 1338, "target": 1327, "value": 180 }, { "source": 1337, "target": 1338, "value": 760 }, { "source": 1336, "target": 615, "value": 1100 }, { "source": 1317, "target": 625, "value": 312 }, { "source": 1335, "target": 1317, "value": 270 }, { "source": 683, "target": 1335, "value": 504 }, { "source": 1334, "target": 1331, "value": 184 }, { "source": 683, "target": 1334, "value": 2124 }, { "source": 601, "target": 1571, "value": 108 }, { "source": 1334, "target": 601, "value": 534 }, { "source": 1334, "target": 63, "value": 990 }, { "source": 1321, "target": 1330, "value": 198 }, { "source": 1323, "target": 1329, "value": 24996 }, { "source": 1321, "target": 1323, "value": 53454 }, { "source": 1323, "target": 1327, "value": 612 }, { "source": 1323, "target": 1328, "value": 272 }, { "source": 1323, "target": 1326, "value": 850 }, { "source": 1323, "target": 1325, "value": 272 }, { "source": 1323, "target": 1324, "value": 306 }, { "source": 1321, "target": 1322, "value": 374 }, { "source": 1319, "target": 679, "value": 816 }, { "source": 683, "target": 1319, "value": 3910 }, { "source": 1319, "target": 50, "value": 2312 }, { "source": 683, "target": 1318, "value": 306 }, { "source": 1315, "target": 1317, "value": 510 }, { "source": 683, "target": 1315, "value": 1088 }, { "source": 1315, "target": 1316, "value": 204 }, { "source": 1569, "target": 610, "value": 8910 }, { "source": 1308, "target": 1567, "value": 514 }, { "source": 1308, "target": 1566, "value": 540 }, { "source": 1314, "target": 295, "value": 612 }, { "source": 1313, "target": 1314, "value": 2376 }, { "source": 1308, "target": 1313, "value": 4890 }, { "source": 1314, "target": 259, "value": 648 }, { "source": 1313, "target": 48, "value": 252 }, { "source": 1313, "target": 42, "value": 396 }, { "source": 1308, "target": 1312, "value": 252 }, { "source": 1310, "target": 1311, "value": 1188 }, { "source": 1308, "target": 1310, "value": 1908 }, { "source": 1308, "target": 1309, "value": 432 }, { "source": 1307, "target": 18, "value": 2166 }, { "source": 1306, "target": 1307, "value": 2268 }, { "source": 1306, "target": 615, "value": 1980 }, { "source": 1304, "target": 1305, "value": 432 }, { "source": 1300, "target": 1301, "value": 1440 }, { "source": 683, "target": 1300, "value": 1656 }, { "source": 683, "target": 1299, "value": 288 }, { "source": 683, "target": 1298, "value": 828 }, { "source": 683, "target": 1297, "value": 432 }, { "source": 1292, "target": 662, "value": 924 }, { "source": 1292, "target": 18, "value": 9576 }, { "source": 1294, "target": 266, "value": 1386 }, { "source": 1293, "target": 1294, "value": 64336 }, { "source": 1294, "target": 268, "value": 3380 }, { "source": 1294, "target": 267, "value": 1386 }, { "source": 1294, "target": 1223, "value": 25368 }, { "source": 675, "target": 295, "value": 714 }, { "source": 1292, "target": 675, "value": 15412 }, { "source": 675, "target": 277, "value": 3152 }, { "source": 676, "target": 273, "value": 4544 }, { "source": 675, "target": 676, "value": 3360 }, { "source": 675, "target": 259, "value": 1284 }, { "source": 1292, "target": 48, "value": 294 }, { "source": 1532, "target": 1191, "value": 65968 }, { "source": 1295, "target": 1296, "value": 90 }, { "source": 1294, "target": 1295, "value": 834 }, { "source": 1295, "target": 662, "value": 88 }, { "source": 160, "target": 682, "value": 10932 }, { "source": 295, "target": 1331, "value": 184 }, { "source": 1288, "target": 295, "value": 526 }, { "source": 1287, "target": 1288, "value": 74944 }, { "source": 101, "target": 1287, "value": 16088 }, { "source": 1586, "target": 101, "value": 6474 }, { "source": 160, "target": 1586, "value": 138142 }, { "source": 668, "target": 263, "value": 72528 }, { "source": 1289, "target": 668, "value": 72918 }, { "source": 1288, "target": 1289, "value": 73498 }, { "source": 285, "target": 286, "value": 1600 }, { "source": 1288, "target": 259, "value": 360 }, { "source": 8, "target": 10, "value": 992 }, { "source": 572, "target": 8, "value": 2828 }, { "source": 571, "target": 572, "value": 3210 }, { "source": 567, "target": 571, "value": 8980 }, { "source": 566, "target": 567, "value": 13670 }, { "source": 565, "target": 566, "value": 15188 }, { "source": 1589, "target": 565, "value": 15524 }, { "source": 1586, "target": 1589, "value": 123842 }, { "source": 8, "target": 9, "value": 1096 }, { "source": 571, "target": 295, "value": 68 }, { "source": 571, "target": 83, "value": 280 }, { "source": 571, "target": 570, "value": 154 }, { "source": 568, "target": 106, "value": 1760 }, { "source": 571, "target": 568, "value": 714 }, { "source": 571, "target": 259, "value": 2676 }, { "source": 1665, "target": 1561, "value": 12996 }, { "source": 1364, "target": 1665, "value": 13620 }, { "source": 1321, "target": 1364, "value": 15616 }, { "source": 1665, "target": 1366, "value": 176 }, { "source": 1364, "target": 615, "value": 1100 }, { "source": 417, "target": 69, "value": 132 }, { "source": 704, "target": 665, "value": 26 }, { "source": 1628, "target": 1629, "value": 15620 }, { "source": 1627, "target": 1628, "value": 16280 }, { "source": 1626, "target": 1627, "value": 22382 }, { "source": 16, "target": 17, "value": 10780 }, { "source": 15, "target": 16, "value": 16240 }, { "source": 1675, "target": 263, "value": 3470 }, { "source": 1672, "target": 1675, "value": 3484 }, { "source": 1269, "target": 1672, "value": 3596 }, { "source": 916, "target": 1269, "value": 18374 }, { "source": 915, "target": 916, "value": 41626 }, { "source": 1544, "target": 915, "value": 18686 }, { "source": 1532, "target": 1544, "value": 22790 }, { "source": 1673, "target": 1674, "value": 30 }, { "source": 1672, "target": 1673, "value": 66 }, { "source": 1668, "target": 1669, "value": 358 }, { "source": 1582, "target": 1668, "value": 326574 }, { "source": 1668, "target": 1040, "value": 4070 }, { "source": 1668, "target": 1075, "value": 12 }, { "source": 1668, "target": 1070, "value": 2050 }, { "source": 1668, "target": 1039, "value": 24 }, { "source": 1562, "target": 665, "value": 7814 }, { "source": 1561, "target": 38, "value": 5328 }, { "source": 1561, "target": 1655, "value": 3840 }, { "source": 1561, "target": 698, "value": 1944 }, { "source": 1561, "target": 697, "value": 108 }, { "source": 1561, "target": 1667, "value": 60 }, { "source": 148, "target": 1287, "value": 59242 }, { "source": 1596, "target": 148, "value": 14366 }, { "source": 1582, "target": 1154, "value": 132 }, { "source": 1664, "target": 1581, "value": 242 }, { "source": 1664, "target": 1580, "value": 352 }, { "source": 1578, "target": 1460, "value": 288 }, { "source": 1664, "target": 1565, "value": 34114 }, { "source": 1650, "target": 295, "value": 374 }, { "source": 701, "target": 1650, "value": 2046 }, { "source": 1650, "target": 259, "value": 396 }, { "source": 24, "target": 25, "value": 352 }, { "source": 701, "target": 24, "value": 836 }, { "source": 1608, "target": 272, "value": 9590 }, { "source": 1607, "target": 1608, "value": 36280 }, { "source": 1606, "target": 1607, "value": 36478 }, { "source": 1532, "target": 1606, "value": 45322 }, { "source": 704, "target": 148, "value": 8372 }, { "source": 1597, "target": 1598, "value": 144 }, { "source": 704, "target": 1597, "value": 18538 }, { "source": 1532, "target": 1649, "value": 110 }, { "source": 1532, "target": 48, "value": 322 }, { "source": 1532, "target": 42, "value": 506 }, { "source": 1626, "target": 255, "value": 1892 }, { "source": 1626, "target": 1648, "value": 110 }, { "source": 1626, "target": 1647, "value": 220 }, { "source": 1626, "target": 48, "value": 154 }, { "source": 1626, "target": 1646, "value": 286 }, { "source": 258, "target": 295, "value": 170 }, { "source": 258, "target": 259, "value": 180 }, { "source": 1644, "target": 1645, "value": 286 }, { "source": 1644, "target": 295, "value": 374 }, { "source": 1644, "target": 259, "value": 396 }, { "source": 1626, "target": 1641, "value": 506 }, { "source": 1626, "target": 15, "value": 3674 }, { "source": 1637, "target": 1640, "value": 550 }, { "source": 1626, "target": 1637, "value": 37092 }, { "source": 1637, "target": 15, "value": 3674 }, { "source": 1638, "target": 295, "value": 748 }, { "source": 1637, "target": 1638, "value": 32450 }, { "source": 1638, "target": 1639, "value": 29480 }, { "source": 1638, "target": 259, "value": 792 }, { "source": 1638, "target": 21, "value": 198 }, { "source": 1626, "target": 42, "value": 242 }, { "source": 1626, "target": 1210, "value": 198 }, { "source": 1636, "target": 487, "value": 1768 }, { "source": 1634, "target": 1636, "value": 2780 }, { "source": 1627, "target": 1634, "value": 4496 }, { "source": 1634, "target": 1635, "value": 836 }, { "source": 1632, "target": 1633, "value": 1628 }, { "source": 1631, "target": 1632, "value": 2200 }, { "source": 1629, "target": 1631, "value": 8448 }, { "source": 1629, "target": 1630, "value": 3828 }, { "source": 1532, "target": 1625, "value": 308 }, { "source": 1532, "target": 1210, "value": 396 }, { "source": 1532, "target": 1175, "value": 242 }, { "source": 1532, "target": 1624, "value": 550 }, { "source": 1622, "target": 1623, "value": 352 }, { "source": 1532, "target": 1622, "value": 1232 }, { "source": 1532, "target": 1621, "value": 396 }, { "source": 1532, "target": 905, "value": 308 }, { "source": 1606, "target": 1620, "value": 572 }, { "source": 1606, "target": 48, "value": 154 }, { "source": 1618, "target": 1619, "value": 2134 }, { "source": 1617, "target": 1618, "value": 2288 }, { "source": 1616, "target": 1617, "value": 2992 }, { "source": 1614, "target": 1616, "value": 4092 }, { "source": 1613, "target": 1614, "value": 4840 }, { "source": 1606, "target": 1613, "value": 5038 }, { "source": 1617, "target": 1615, "value": 242 }, { "source": 1614, "target": 1615, "value": 242 }, { "source": 1608, "target": 295, "value": 374 }, { "source": 1608, "target": 48, "value": 154 }, { "source": 1612, "target": 688, "value": 594 }, { "source": 1608, "target": 1612, "value": 1056 }, { "source": 1608, "target": 1611, "value": 352 }, { "source": 1608, "target": 1610, "value": 308 }, { "source": 1608, "target": 685, "value": 1578 }, { "source": 1608, "target": 724, "value": 19832 }, { "source": 1609, "target": 42, "value": 242 }, { "source": 1608, "target": 1609, "value": 1408 }, { "source": 1609, "target": 259, "value": 396 }, { "source": 1606, "target": 42, "value": 242 }, { "source": 1606, "target": 1215, "value": 154 }, { "source": 1605, "target": 1218, "value": 1012 }, { "source": 1532, "target": 1605, "value": 2596 }, { "source": 1605, "target": 4, "value": 110 }, { "source": 1605, "target": 417, "value": 418 }, { "source": 1532, "target": 1604, "value": 528 }, { "source": 1532, "target": 894, "value": 704 }, { "source": 1603, "target": 69, "value": 132 }, { "source": 1602, "target": 1603, "value": 572 }, { "source": 1532, "target": 1602, "value": 726 }, { "source": 1601, "target": 48, "value": 308 }, { "source": 1532, "target": 1601, "value": 1518 }, { "source": 1601, "target": 42, "value": 484 }, { "source": 1599, "target": 1600, "value": 440 }, { "source": 1532, "target": 1599, "value": 1342 }, { "source": 1599, "target": 48, "value": 154 }, { "source": 1599, "target": 42, "value": 242 }, { "source": 274, "target": 279, "value": 180 }, { "source": 263, "target": 1198, "value": 252 }, { "source": 265, "target": 269, "value": 52 }, { "source": 1596, "target": 1597, "value": 193648 }, { "source": 145, "target": 69, "value": 792 }, { "source": 1532, "target": 1595, "value": 600 }, { "source": 1594, "target": 540, "value": 2904 }, { "source": 1593, "target": 1594, "value": 11976 }, { "source": 1532, "target": 1593, "value": 12960 }, { "source": 1594, "target": 539, "value": 96 }, { "source": 1594, "target": 535, "value": 552 }, { "source": 1594, "target": 530, "value": 6888 }, { "source": 1551, "target": 69, "value": 144 }, { "source": 1532, "target": 1551, "value": 408 }, { "source": 1532, "target": 1550, "value": 552 }, { "source": 1532, "target": 101, "value": 216 }, { "source": 1191, "target": 69, "value": 600 }, { "source": 1532, "target": 578, "value": 1824 }, { "source": 1269, "target": 922, "value": 464 }, { "source": 1269, "target": 48, "value": 224 }, { "source": 1269, "target": 42, "value": 352 }, { "source": 919, "target": 131, "value": 234 }, { "source": 1269, "target": 919, "value": 9920 }, { "source": 98, "target": 100, "value": 484 }, { "source": 919, "target": 98, "value": 1664 }, { "source": 98, "target": 99, "value": 1716 }, { "source": 919, "target": 920, "value": 386 }, { "source": 919, "target": 69, "value": 156 }, { "source": 1360, "target": 1361, "value": 1612 }, { "source": 1359, "target": 1360, "value": 4056 }, { "source": 919, "target": 1359, "value": 5850 }, { "source": 1544, "target": 1220, "value": 264 }, { "source": 1544, "target": 1548, "value": 264 }, { "source": 1544, "target": 1547, "value": 408 }, { "source": 1545, "target": 1546, "value": 792 }, { "source": 1544, "target": 1545, "value": 1992 }, { "source": 1541, "target": 295, "value": 408 }, { "source": 1532, "target": 1541, "value": 13182 }, { "source": 1542, "target": 295, "value": 408 }, { "source": 1541, "target": 1542, "value": 10872 }, { "source": 1542, "target": 259, "value": 432 }, { "source": 1543, "target": 16, "value": 3360 }, { "source": 1542, "target": 1543, "value": 4008 }, { "source": 1542, "target": 15, "value": 4008 }, { "source": 1541, "target": 259, "value": 432 }, { "source": 1541, "target": 665, "value": 26 }, { "source": 141, "target": 232, "value": 264 }, { "source": 1541, "target": 141, "value": 140 }, { "source": 141, "target": 69, "value": 792 }, { "source": 1539, "target": 15, "value": 4008 }, { "source": 1532, "target": 1539, "value": 5736 }, { "source": 1539, "target": 665, "value": 312 }, { "source": 14, "target": 22, "value": 1416 }, { "source": 1532, "target": 14, "value": 5832 }, { "source": 14, "target": 15, "value": 4008 }, { "source": 1534, "target": 101, "value": 216 }, { "source": 1532, "target": 1534, "value": 2186 }, { "source": 1534, "target": 665, "value": 410 }, { "source": 1534, "target": 1191, "value": 432 }, { "source": 878, "target": 879, "value": 576 }, { "source": 1533, "target": 878, "value": 768 }, { "source": 1532, "target": 1533, "value": 984 }, { "source": 1590, "target": 1591, "value": 936 }, { "source": 1586, "target": 1590, "value": 1272 }, { "source": 576, "target": 69, "value": 864 }, { "source": 575, "target": 576, "value": 101926 }, { "source": 1589, "target": 575, "value": 103942 }, { "source": 1589, "target": 585, "value": 816 }, { "source": 576, "target": 579, "value": 864 }, { "source": 576, "target": 48, "value": 476 }, { "source": 576, "target": 578, "value": 2602 }, { "source": 576, "target": 42, "value": 748 }, { "source": 576, "target": 577, "value": 576 }, { "source": 567, "target": 295, "value": 170 }, { "source": 569, "target": 83, "value": 1484 }, { "source": 567, "target": 569, "value": 10142 }, { "source": 569, "target": 570, "value": 814 }, { "source": 567, "target": 568, "value": 4038 }, { "source": 567, "target": 259, "value": 1116 }, { "source": 1586, "target": 1588, "value": 480 }, { "source": 1587, "target": 70, "value": 1586 }, { "source": 1586, "target": 1587, "value": 1970 }, { "source": 1586, "target": 1191, "value": 432 }, { "source": 71, "target": 48, "value": 336 }, { "source": 67, "target": 71, "value": 109488 }, { "source": 71, "target": 1576, "value": 26 }, { "source": 1562, "target": 1289, "value": 428 }, { "source": 90, "target": 98, "value": 1152 }, { "source": 71, "target": 90, "value": 7538 }, { "source": 90, "target": 48, "value": 126 }, { "source": 90, "target": 97, "value": 90 }, { "source": 90, "target": 96, "value": 432 }, { "source": 90, "target": 95, "value": 180 }, { "source": 90, "target": 42, "value": 198 }, { "source": 91, "target": 69, "value": 108 }, { "source": 90, "target": 91, "value": 3020 }, { "source": 93, "target": 94, "value": 734 }, { "source": 92, "target": 93, "value": 842 }, { "source": 91, "target": 92, "value": 1472 }, { "source": 71, "target": 72, "value": 1024 }, { "source": 725, "target": 671, "value": 28 }, { "source": 1664, "target": 832, "value": 2564 }, { "source": 1582, "target": 1663, "value": 120 }, { "source": 1460, "target": 832, "value": 2618 }, { "source": 1662, "target": 1460, "value": 2956 }, { "source": 1582, "target": 1662, "value": 5684 }, { "source": 1662, "target": 832, "value": 2366 }, { "source": 1084, "target": 1095, "value": 3640 }, { "source": 1084, "target": 1094, "value": 6320 }, { "source": 1084, "target": 1093, "value": 11920 }, { "source": 1086, "target": 1085, "value": 102400 }, { "source": 1087, "target": 1086, "value": 83200 }, { "source": 1088, "target": 1087, "value": 86400 }, { "source": 1089, "target": 1088, "value": 88000 }, { "source": 1090, "target": 1089, "value": 88800 }, { "source": 1091, "target": 1090, "value": 101014 }, { "source": 1092, "target": 1091, "value": 101214 }, { "source": 1084, "target": 1092, "value": 190814 }, { "source": 1087, "target": 1085, "value": 51200 }, { "source": 1088, "target": 1086, "value": 41600 }, { "source": 1088, "target": 1085, "value": 25600 }, { "source": 1089, "target": 1087, "value": 43200 }, { "source": 1089, "target": 1086, "value": 20800 }, { "source": 1089, "target": 1085, "value": 12800 }, { "source": 1090, "target": 1088, "value": 44000 }, { "source": 1090, "target": 1087, "value": 21600 }, { "source": 1090, "target": 1086, "value": 10400 }, { "source": 1090, "target": 1085, "value": 18214 }, { "source": 1091, "target": 1089, "value": 44400 }, { "source": 1091, "target": 1088, "value": 22000 }, { "source": 1091, "target": 1087, "value": 10800 }, { "source": 1091, "target": 1086, "value": 5200 }, { "source": 1091, "target": 1085, "value": 3200 }, { "source": 1092, "target": 1090, "value": 44600 }, { "source": 1092, "target": 1089, "value": 22200 }, { "source": 1092, "target": 1088, "value": 11000 }, { "source": 1092, "target": 1087, "value": 5400 }, { "source": 1092, "target": 1086, "value": 2600 }, { "source": 1092, "target": 1085, "value": 1600 }, { "source": 1084, "target": 1091, "value": 89400 }, { "source": 1084, "target": 1090, "value": 44600 }, { "source": 1084, "target": 1089, "value": 22200 }, { "source": 1084, "target": 1088, "value": 11000 }, { "source": 1084, "target": 1087, "value": 5400 }, { "source": 1084, "target": 1086, "value": 2600 }, { "source": 1084, "target": 1085, "value": 1600 }, { "source": 1661, "target": 832, "value": 2084 }, { "source": 1582, "target": 1661, "value": 2854 }, { "source": 1660, "target": 832, "value": 2262 }, { "source": 1582, "target": 1660, "value": 2720 }, { "source": 31, "target": 27, "value": 116 }, { "source": 1681, "target": 31, "value": 208 }, { "source": 701, "target": 1681, "value": 912 }, { "source": 1681, "target": 1683, "value": 22 }, { "source": 7, "target": 8, "value": 396 }, { "source": 6, "target": 7, "value": 432 }, { "source": 5, "target": 6, "value": 462 }, { "source": 1682, "target": 5, "value": 478 }, { "source": 1681, "target": 1682, "value": 590 }, { "source": 1682, "target": 48, "value": 14 }, { "source": 1682, "target": 42, "value": 22 }, { "source": 84, "target": 85, "value": 2464 }, { "source": 81, "target": 84, "value": 5104 }, { "source": 79, "target": 81, "value": 39518 }, { "source": 76, "target": 79, "value": 10740 }, { "source": 81, "target": 83, "value": 2210 }, { "source": 81, "target": 82, "value": 2876 }, { "source": 81, "target": 80, "value": 2464 }, { "source": 79, "target": 80, "value": 1760 }, { "source": 76, "target": 78, "value": 500 }, { "source": 76, "target": 77, "value": 220 }, { "source": 1027, "target": 1038, "value": 633092 }, { "source": 1277, "target": 1278, "value": 7280 }, { "source": 1263, "target": 1277, "value": 7368 }, { "source": 1261, "target": 1262, "value": 48 }, { "source": 1260, "target": 1261, "value": 144 }, { "source": 1038, "target": 1040, "value": 37462 }, { "source": 460, "target": 341, "value": 1232 }, { "source": 334, "target": 295, "value": 340 }, { "source": 334, "target": 259, "value": 360 }, { "source": 1038, "target": 1354, "value": 27494 }, { "source": 916, "target": 917, "value": 21718 }, { "source": 160, "target": 915, "value": 24364 }, { "source": 917, "target": 1362, "value": 19990 }, { "source": 1362, "target": 704, "value": 19830 }, { "source": 1364, "target": 1371, "value": 64 }, { "source": 1365, "target": 1367, "value": 32 }, { "source": 1364, "target": 1365, "value": 396 }, { "source": 1365, "target": 1339, "value": 38 }, { "source": 1369, "target": 1370, "value": 50 }, { "source": 1368, "target": 1369, "value": 78 }, { "source": 1365, "target": 1368, "value": 96 }, { "source": 1365, "target": 1307, "value": 126 }, { "source": 1365, "target": 1366, "value": 22 }, { "source": 1362, "target": 48, "value": 14 }, { "source": 1362, "target": 1363, "value": 62 }, { "source": 918, "target": 919, "value": 900 }, { "source": 917, "target": 918, "value": 1528 }, { "source": 1354, "target": 1358, "value": 22 }, { "source": 1354, "target": 1357, "value": 12 }, { "source": 1354, "target": 832, "value": 1972 }, { "source": 1349, "target": 1353, "value": 14 }, { "source": 1038, "target": 1349, "value": 162 }, { "source": 1352, "target": 1351, "value": 20 }, { "source": 1349, "target": 1352, "value": 62 }, { "source": 1350, "target": 1351, "value": 20 }, { "source": 1349, "target": 1350, "value": 46 }, { "source": 1038, "target": 1226, "value": 260 }, { "source": 1242, "target": 1243, "value": 154 }, { "source": 1241, "target": 1242, "value": 770 }, { "source": 1240, "target": 1241, "value": 1276 }, { "source": 1228, "target": 1240, "value": 1134 }, { "source": 1038, "target": 1228, "value": 28538 }, { "source": 1242, "target": 1239, "value": 132 }, { "source": 1241, "target": 1239, "value": 132 }, { "source": 1240, "target": 1239, "value": 132 }, { "source": 1167, "target": 1168, "value": 211106 }, { "source": 1151, "target": 1167, "value": 211810 }, { "source": 1038, "target": 1151, "value": 511842 }, { "source": 453, "target": 455, "value": 1116 }, { "source": 141, "target": 453, "value": 3960 }, { "source": 553, "target": 141, "value": 1472 }, { "source": 551, "target": 553, "value": 7050 }, { "source": 550, "target": 551, "value": 25298 }, { "source": 549, "target": 550, "value": 25606 }, { "source": 546, "target": 549, "value": 34362 }, { "source": 160, "target": 546, "value": 80002 }, { "source": 453, "target": 454, "value": 374 }, { "source": 1151, "target": 1164, "value": 101342 }, { "source": 1159, "target": 345, "value": 264 }, { "source": 1151, "target": 1159, "value": 10366 }, { "source": 1159, "target": 301, "value": 814 }, { "source": 174, "target": 1116, "value": 3146 }, { "source": 1159, "target": 174, "value": 5966 }, { "source": 1151, "target": 1157, "value": 101678 }, { "source": 1151, "target": 1154, "value": 120 }, { "source": 1099, "target": 1116, "value": 13318 }, { "source": 1096, "target": 1099, "value": 8222 }, { "source": 1038, "target": 1096, "value": 7248 }, { "source": 1348, "target": 48, "value": 14 }, { "source": 453, "target": 1348, "value": 464 }, { "source": 1179, "target": 453, "value": 2028 }, { "source": 1174, "target": 1179, "value": 3372 }, { "source": 1173, "target": 1174, "value": 74724 }, { "source": 1172, "target": 1173, "value": 129490 }, { "source": 1171, "target": 1172, "value": 129688 }, { "source": 160, "target": 1171, "value": 130172 }, { "source": 1348, "target": 455, "value": 72 }, { "source": 1348, "target": 42, "value": 22 }, { "source": 546, "target": 148, "value": 20378 }, { "source": 1187, "target": 141, "value": 1728 }, { "source": 1174, "target": 1187, "value": 2058 }, { "source": 109, "target": 141, "value": 1626 }, { "source": 71, "target": 109, "value": 9244 }, { "source": 307, "target": 145, "value": 20986 }, { "source": 306, "target": 307, "value": 81398 }, { "source": 305, "target": 306, "value": 82058 }, { "source": 160, "target": 305, "value": 82258 }, { "source": 551, "target": 141, "value": 2282 }, { "source": 147, "target": 1264, "value": 752 }, { "source": 146, "target": 147, "value": 3594 }, { "source": 553, "target": 146, "value": 3510 }, { "source": 126, "target": 567, "value": 13204 }, { "source": 125, "target": 126, "value": 14240 }, { "source": 124, "target": 125, "value": 14516 }, { "source": 71, "target": 124, "value": 18200 }, { "source": 546, "target": 145, "value": 21688 }, { "source": 307, "target": 148, "value": 16718 }, { "source": 265, "target": 1223, "value": 3724 }, { "source": 318, "target": 141, "value": 3392 }, { "source": 312, "target": 318, "value": 32942 }, { "source": 307, "target": 312, "value": 41694 }, { "source": 1278, "target": 832, "value": 3310 }, { "source": 1229, "target": 1263, "value": 7496 }, { "source": 1228, "target": 1229, "value": 26896 }, { "source": 832, "target": 818, "value": 26 }, { "source": 1270, "target": 832, "value": 2100 }, { "source": 1259, "target": 1270, "value": 11554 }, { "source": 1258, "target": 1259, "value": 11770 }, { "source": 1255, "target": 1258, "value": 11890 }, { "source": 1251, "target": 1255, "value": 12474 }, { "source": 1250, "target": 1251, "value": 13050 }, { "source": 1249, "target": 1250, "value": 13162 }, { "source": 1248, "target": 1249, "value": 13274 }, { "source": 1245, "target": 1248, "value": 13394 }, { "source": 1229, "target": 1245, "value": 13922 }, { "source": 1221, "target": 285, "value": 1904 }, { "source": 1173, "target": 1221, "value": 52060 }, { "source": 1221, "target": 271, "value": 16842 }, { "source": 109, "target": 146, "value": 3186 }, { "source": 1229, "target": 1260, "value": 248 }, { "source": 1229, "target": 1244, "value": 96 }, { "source": 1256, "target": 1257, "value": 144 }, { "source": 1255, "target": 1256, "value": 360 }, { "source": 1253, "target": 1254, "value": 112 }, { "source": 1252, "target": 1253, "value": 256 }, { "source": 1251, "target": 1252, "value": 400 }, { "source": 1246, "target": 1247, "value": 144 }, { "source": 1245, "target": 1246, "value": 336 }, { "source": 1229, "target": 1240, "value": 648 }, { "source": 1229, "target": 1226, "value": 104 }, { "source": 1229, "target": 1239, "value": 48 }, { "source": 1238, "target": 1096, "value": 1678 }, { "source": 1237, "target": 1238, "value": 1878 }, { "source": 1229, "target": 1237, "value": 2006 }, { "source": 1238, "target": 1235, "value": 48 }, { "source": 1234, "target": 1235, "value": 96 }, { "source": 1233, "target": 1234, "value": 240 }, { "source": 1231, "target": 1233, "value": 384 }, { "source": 1230, "target": 1231, "value": 656 }, { "source": 1229, "target": 1230, "value": 1592 }, { "source": 1231, "target": 1232, "value": 96 }, { "source": 1230, "target": 1226, "value": 416 }, { "source": 1230, "target": 1236, "value": 104 }, { "source": 1038, "target": 1227, "value": 468 }, { "source": 1221, "target": 295, "value": 374 }, { "source": 1221, "target": 48, "value": 154 }, { "source": 1221, "target": 290, "value": 1100 }, { "source": 1221, "target": 42, "value": 242 }, { "source": 1197, "target": 642, "value": 3146 }, { "source": 1225, "target": 1197, "value": 2832 }, { "source": 1221, "target": 1225, "value": 3360 }, { "source": 1221, "target": 1198, "value": 1258 }, { "source": 1221, "target": 265, "value": 24494 }, { "source": 1221, "target": 259, "value": 396 }, { "source": 1173, "target": 1220, "value": 242 }, { "source": 1173, "target": 913, "value": 638 }, { "source": 1174, "target": 1219, "value": 484 }, { "source": 1217, "target": 1218, "value": 506 }, { "source": 1174, "target": 1217, "value": 660 }, { "source": 1174, "target": 101, "value": 198 }, { "source": 1214, "target": 1216, "value": 352 }, { "source": 1174, "target": 1214, "value": 3168 }, { "source": 1214, "target": 48, "value": 154 }, { "source": 1214, "target": 42, "value": 242 }, { "source": 1214, "target": 1215, "value": 154 }, { "source": 1174, "target": 1213, "value": 396 }, { "source": 1174, "target": 1212, "value": 154 }, { "source": 1174, "target": 48, "value": 308 }, { "source": 1174, "target": 1211, "value": 374 }, { "source": 931, "target": 932, "value": 352 }, { "source": 1174, "target": 931, "value": 726 }, { "source": 1174, "target": 42, "value": 498 }, { "source": 1174, "target": 1210, "value": 198 }, { "source": 1174, "target": 1209, "value": 110 }, { "source": 1174, "target": 1208, "value": 264 }, { "source": 1174, "target": 1207, "value": 352 }, { "source": 1206, "target": 665, "value": 286 }, { "source": 1174, "target": 1206, "value": 5258 }, { "source": 1206, "target": 141, "value": 1298 }, { "source": 1205, "target": 773, "value": 814 }, { "source": 1174, "target": 1205, "value": 2662 }, { "source": 1174, "target": 1204, "value": 440 }, { "source": 1174, "target": 1203, "value": 418 }, { "source": 1174, "target": 1202, "value": 132 }, { "source": 1199, "target": 1201, "value": 1056 }, { "source": 1174, "target": 1199, "value": 1848 }, { "source": 1199, "target": 1200, "value": 264 }, { "source": 1194, "target": 295, "value": 374 }, { "source": 1174, "target": 1194, "value": 13332 }, { "source": 1194, "target": 1198, "value": 726 }, { "source": 1194, "target": 259, "value": 396 }, { "source": 1196, "target": 295, "value": 374 }, { "source": 1194, "target": 1196, "value": 9042 }, { "source": 280, "target": 1197, "value": 2596 }, { "source": 1196, "target": 280, "value": 3168 }, { "source": 1196, "target": 276, "value": 1980 }, { "source": 1196, "target": 676, "value": 1760 }, { "source": 1196, "target": 259, "value": 396 }, { "source": 1195, "target": 662, "value": 22 }, { "source": 1194, "target": 1195, "value": 1584 }, { "source": 1174, "target": 1193, "value": 176 }, { "source": 1192, "target": 48, "value": 154 }, { "source": 1174, "target": 1192, "value": 902 }, { "source": 1192, "target": 42, "value": 242 }, { "source": 1174, "target": 1191, "value": 396 }, { "source": 1190, "target": 623, "value": 242 }, { "source": 1174, "target": 1190, "value": 682 }, { "source": 1189, "target": 608, "value": 396 }, { "source": 1174, "target": 1189, "value": 1430 }, { "source": 1174, "target": 1188, "value": 176 }, { "source": 1174, "target": 402, "value": 330 }, { "source": 1174, "target": 1186, "value": 638 }, { "source": 1174, "target": 1185, "value": 176 }, { "source": 1174, "target": 1184, "value": 264 }, { "source": 1182, "target": 1183, "value": 726 }, { "source": 1174, "target": 1182, "value": 1298 }, { "source": 1174, "target": 1181, "value": 154 }, { "source": 1180, "target": 232, "value": 132 }, { "source": 1174, "target": 1180, "value": 2266 }, { "source": 1180, "target": 141, "value": 1298 }, { "source": 1174, "target": 79, "value": 16172 }, { "source": 81, "target": 10, "value": 1364 }, { "source": 81, "target": 48, "value": 154 }, { "source": 81, "target": 1062, "value": 1894 }, { "source": 81, "target": 259, "value": 396 }, { "source": 79, "target": 69, "value": 528 }, { "source": 1179, "target": 69, "value": 132 }, { "source": 1174, "target": 1178, "value": 176 }, { "source": 1176, "target": 1177, "value": 132 }, { "source": 1174, "target": 1176, "value": 286 }, { "source": 1174, "target": 1175, "value": 242 }, { "source": 554, "target": 735, "value": 528 }, { "source": 551, "target": 554, "value": 14030 }, { "source": 147, "target": 460, "value": 2116 }, { "source": 340, "target": 147, "value": 5160 }, { "source": 338, "target": 340, "value": 13262 }, { "source": 318, "target": 338, "value": 14382 }, { "source": 333, "target": 334, "value": 1260 }, { "source": 331, "target": 333, "value": 2260 }, { "source": 330, "target": 331, "value": 3080 }, { "source": 318, "target": 330, "value": 3220 }, { "source": 1043, "target": 1061, "value": 2512 }, { "source": 1145, "target": 1043, "value": 7872 }, { "source": 1142, "target": 1145, "value": 10080 }, { "source": 1038, "target": 1142, "value": 10368 }, { "source": 1149, "target": 1059, "value": 336 }, { "source": 1145, "target": 1149, "value": 896 }, { "source": 1146, "target": 1147, "value": 352 }, { "source": 1145, "target": 1146, "value": 704 }, { "source": 1140, "target": 1099, "value": 7736 }, { "source": 1137, "target": 1140, "value": 7992 }, { "source": 1038, "target": 1137, "value": 8072 }, { "source": 71, "target": 1135, "value": 540 }, { "source": 338, "target": 342, "value": 100 }, { "source": 340, "target": 341, "value": 7598 }, { "source": 338, "target": 339, "value": 280 }, { "source": 331, "target": 332, "value": 360 }, { "source": 316, "target": 317, "value": 100 }, { "source": 313, "target": 316, "value": 240 }, { "source": 313, "target": 314, "value": 5592 }, { "source": 1074, "target": 913, "value": 116 }, { "source": 160, "target": 1074, "value": 156 }, { "source": 110, "target": 79, "value": 24988 }, { "source": 71, "target": 110, "value": 30738 }, { "source": 918, "target": 922, "value": 26 }, { "source": 918, "target": 48, "value": 14 }, { "source": 918, "target": 921, "value": 64 }, { "source": 918, "target": 42, "value": 44 }, { "source": 160, "target": 914, "value": 312 }, { "source": 558, "target": 559, "value": 3300 }, { "source": 549, "target": 558, "value": 3960 }, { "source": 554, "target": 340, "value": 4356 }, { "source": 554, "target": 148, "value": 198 }, { "source": 554, "target": 146, "value": 2046 }, { "source": 554, "target": 736, "value": 1078 }, { "source": 554, "target": 145, "value": 440 }, { "source": 553, "target": 148, "value": 198 }, { "source": 553, "target": 145, "value": 440 }, { "source": 81, "target": 86, "value": 330 }, { "source": 110, "target": 78, "value": 1150 }, { "source": 110, "target": 77, "value": 506 }, { "source": 110, "target": 111, "value": 1472 }, { "source": 109, "target": 148, "value": 198 }, { "source": 109, "target": 48, "value": 154 }, { "source": 109, "target": 42, "value": 242 }, { "source": 109, "target": 145, "value": 440 }, { "source": 142, "target": 143, "value": 924 }, { "source": 109, "target": 142, "value": 1276 }, { "source": 71, "target": 42, "value": 2222 }, { "source": 318, "target": 335, "value": 1280 }, { "source": 160, "target": 38, "value": 56586 }, { "source": 124, "target": 83, "value": 924 }, { "source": 117, "target": 119, "value": 1058 }, { "source": 115, "target": 117, "value": 4922 }, { "source": 114, "target": 115, "value": 10350 }, { "source": 117, "target": 118, "value": 874 }, { "source": 117, "target": 106, "value": 920 }, { "source": 115, "target": 116, "value": 3680 }, { "source": 109, "target": 69, "value": 276 }, { "source": 554, "target": 339, "value": 308 }, { "source": 551, "target": 552, "value": 286 }, { "source": 549, "target": 343, "value": 550 }, { "source": 549, "target": 344, "value": 440 }, { "source": 549, "target": 337, "value": 242 }, { "source": 549, "target": 557, "value": 154 }, { "source": 549, "target": 335, "value": 946 }, { "source": 548, "target": 310, "value": 110 }, { "source": 547, "target": 548, "value": 396 }, { "source": 546, "target": 547, "value": 550 }, { "source": 546, "target": 70, "value": 626 }, { "source": 324, "target": 327, "value": 480 }, { "source": 321, "target": 324, "value": 2280 }, { "source": 319, "target": 321, "value": 4320 }, { "source": 318, "target": 319, "value": 5480 }, { "source": 308, "target": 311, "value": 100 }, { "source": 307, "target": 308, "value": 980 }, { "source": 318, "target": 344, "value": 200 }, { "source": 318, "target": 343, "value": 500 }, { "source": 318, "target": 337, "value": 220 }, { "source": 335, "target": 336, "value": 180 }, { "source": 324, "target": 326, "value": 640 }, { "source": 324, "target": 325, "value": 20 }, { "source": 322, "target": 323, "value": 380 }, { "source": 321, "target": 322, "value": 880 }, { "source": 312, "target": 313, "value": 6792 }, { "source": 314, "target": 315, "value": 4392 }, { "source": 309, "target": 310, "value": 100 }, { "source": 308, "target": 309, "value": 260 }, { "source": 67, "target": 101, "value": 576 }, { "source": 67, "target": 70, "value": 3510 }, { "source": 67, "target": 69, "value": 348 }, { "source": 67, "target": 68, "value": 820 }, { "source": 61, "target": 39, "value": 22208 }, { "source": 56, "target": 58, "value": 1408 }, { "source": 163, "target": 39, "value": 52774 }, { "source": 160, "target": 163, "value": 61134 }, { "source": 71, "target": 97, "value": 230 }, { "source": 71, "target": 95, "value": 460 }, { "source": 71, "target": 114, "value": 10810 }, { "source": 61, "target": 40, "value": 24640 }, { "source": 74, "target": 48, "value": 154 }, { "source": 71, "target": 74, "value": 15536 }, { "source": 74, "target": 89, "value": 100 }, { "source": 74, "target": 42, "value": 220 }, { "source": 75, "target": 88, "value": 800 }, { "source": 74, "target": 75, "value": 13920 }, { "source": 75, "target": 76, "value": 12760 }, { "source": 71, "target": 73, "value": 180 }],
    },
};
// A Scale to generate colors for each function name.
var function_color = d3.scale.category20();


// # Create Butterly visualization

// Create `Butterfly` visualization object
var cfg = new c3.Butterfly({
    // Bind to the DOM and set height.
    anchor: '#cfg_butterfly',
    height: 600,

    // Link to control flow graph `functions` and `links` **data**
    data: cfg_datasets['libpthread'].functions,
    links: cfg_datasets['libpthread'].links,
    
    // Define unique **key** accessor for functions
    key: (func) => func.id,

    // **Align** CFG to start on the `left`
    align: 'left',

    // **Style** nodes based on the function name and create tooltips.
    // **Animate** transitions for all of the nodes and links.
    node_options: {
        title: (func) => func.name,
        animate: true,
        duration: 2000,
    },
    rect_options: {
        styles: {
            fill: (func) => function_color(func.name),
            stroke: 'black',
        },
        animate: true,
        duration: 2000,
    },
    link_options: {
        // A poor-performing method of constructing a tooltip with function names.
        // A look-up hash could be used.  The sankey object could be extended with this
        // functionality if requested for relatively little additional space cost.
        title: (link) => cfg.data.filter((f) => f.id == link.source)[0].name + " → " + cfg.data.filter((f) => f.id == link.target)[0].name,
        animate: true,
        duration: 2000,
    },
    path_options: {
        animate: true,
        duration: 2000,
    },

    // Add text **labels** for each node
    node_label_options: {
        text: (func) => func.name,
        styles: {
            'font-weight': 'bold',
            'font-size': 'x-small',
        },
        orientation: 'vertical',
        animate: true,
        duration: 2000,
    },
});
cfg.render();


// ## Extend dynamic chart behavior

// Resize the control flow graph when the window is resized.
window.onresize = function () { cfg.resize(); };


// Select example **data set**
document.getElementById('dataset').addEventListener('change', function () {
    cfg.data = cfg_datasets[this.value].functions;
    cfg.links = cfg_datasets[this.value].links;
    cfg.redraw();
});

// Set **Depth of Field**
document.getElementById('depth_of_field').addEventListener('change', function () {
    cfg.depth_of_field = +this.value;
    cfg.redraw();
});
